var Demux = function()
{
	var demux = {};
	var debug = 0;

	var pipeline = null;
	var subtypeSequencingRun = null;
	var annotationTypePipeline = null;
	var seqRunsAreValid = false;
	var clusterIsValid = false;
	var slotsIsValid = false;
	var newSeqRuns;
	var allLibs = [];
	
	// Page initialization
	demux.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectSeqRuns', demux.selectSeqencingRuns);
		Events.addEventHandler('seqRuns', 'base-selected', demux.setSeqRunCallback);
		Events.addEventHandler('seqRuns', 'change', demux.seqRunsOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', demux.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', demux.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', demux.validateStep2);
		Events.addEventHandler('clusters', 'change', demux.clusterOnChange);
		Events.addEventHandler('slots', 'change', demux.slotsOnChange);
		Events.addEventHandler('slots', 'keypress', Events.integerOnly);
		Events.addEventHandler('demuxSoftware', 'change', demux.softwareOnChange);
		Events.addEventHandler('mergeSoftware', 'change', demux.softwareOnChange);
		Buttons.addClickHandler('donotuseinfo', demux.showDoNotUseInfo);
		Buttons.addClickHandler('libsAll', demux.selectLibsAll);
		Buttons.addClickHandler('libsExceptDoNotUse', demux.selectLibsAllExceptDoNotUse);
		Buttons.addClickHandler('libsSelected', demux.selectLibsForDemux);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', demux.submit);
		

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../DemuxMerge.servlet?ID='+App.getSessionId();
		url += '&cmd=GetConfirmedSequencingRuns&pipeline='+encodeURIComponent(Data.get('page-data', 'pipeline'));
		Wizard.asyncJsonRequest(url, demux.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	demux.initializeStep1 = function(response)
	{
		pipeline = response.pipeline; // We need the 'name' value here to be able to filter in "Select manually"
		var seqRuns = response.sequencingRuns;
		seqRuns.sort(demux.sortByPool);
		var frm = document.forms['reggie'];	

		if (seqRuns.length > 0)
		{
			for (var runNo=0; runNo < seqRuns.length; runNo++)
			{
				var seqRun = seqRuns[runNo];
				var option = demux.getOptionForSeqRun(seqRun);
				frm.seqRuns.options[frm.seqRuns.length] = option;
			}
			demux.seqRunsOnChange();
		}
		else
		{
			Wizard.setInputStatus('seqRuns', 'invalid', 'No new sequencing runs available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.seqRuns.focus();
	}
	
	demux.validateStep1 = function(event)
	{
		if (!seqRunsAreValid) event.preventDefault();
	}
	
	/**
		Sort by the name of the first pool.
	*/
	demux.sortByPool = function(a, b)
	{
		var p1 = a.flowCell.pools[0].name;
		var p2 = b.flowCell.pools[0].name;
		if (p1 < p2) return -1;
		if (p1 > p2) return 1;
		if (a.name < b.name) return -1;
		if (a.name > b.name) return 1;
		return a.id - b.id;
	}
	
	demux.getOptionForSeqRun = function(seqRun)
	{
		var flowCell = seqRun.flowCell;
		var name = seqRun.name+': ';
		var numPools = flowCell.pools.length;
		var firstPoolNum = demux.getPoolNum(flowCell.pools[0].name);
		var lastPoolNum = demux.getPoolNum(flowCell.pools[flowCell.pools.length-1].name);

		if (lastPoolNum - firstPoolNum == numPools - 1)
		{
			if (numPools > 1)
			{
				// Display: PoolN -- PoolY
				name += flowCell.pools[0].name + ' — ' + flowCell.pools[flowCell.pools.length-1].name;
			}
			else
			{
				name += flowCell.pools[0].name;
			}
		}
		else
		{
			// Display: PoolN + x more...
			name += flowCell.pools[0].name + ' + ' + (numPools-1) + ' more...';
		}
		
		var opt = [];
		opt[opt.length] = flowCell.libraries.length + ' libraries';
		if (flowCell.FlowCellType) opt[opt.length] = flowCell.FlowCellType;
		opt[opt.length] = seqRun.pipeline;
		name += ' [' + opt.join('; ') + ']';

		if (seqRun.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		// Count the number of libraries marked with DoNotUse
		var numDoNotUse = 0;
		for (var libNo = 0; libNo < flowCell.libraries.length; libNo++)
		{
			if (flowCell.libraries[libNo].DO_NOT_USE) numDoNotUse++;
		}
		flowCell.numDoNotUse = numDoNotUse;
		
		// Tooltip is always all pools
		var title = '';
		for (var poolNo=0; poolNo < numPools; poolNo++)
		{
			var pool = flowCell.pools[poolNo];
			if (poolNo > 0) title += ', ';
			title += pool.name;
		}

		var option = new Option(name, seqRun.id);
		option.title = title;
		option.seqRun = seqRun;	
		return option;
	}

	demux.getPoolNum = function(poolName)
	{
		var num = poolName.match(/Pool(\d+)/);
		return num ? parseInt(num[1], 10) : null;
	}

	demux.selectSeqencingRuns = function()
	{
		var frm = document.forms['reggie'];
		if (frm.seqRuns.disabled) return;
		
		if (subtypeSequencingRun == null)
		{
			subtypeSequencingRun = Reggie.getSubtypeInfo('SEQUENCING_RUN');
		}
		if (annotationTypePipeline == null)
		{
			annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		}
		
		newSeqRuns = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeSequencingRun.id;
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'='+encodeURIComponent(pipeline.name);
		Dialogs.selectItem('DERIVEDBIOASSAY', 'seqRuns', 1, url);
	}

	demux.setSeqRunCallback = function(event)
	{
		var frm = document.forms['reggie'];
		var seqRunList = frm.seqRuns;

		var isNew = true;
		for (var i = 0; i < seqRunList.length; i++)
		{
			if (seqRunList[i].value == event.detail.id)
			{
				seqRunList[i].selected = true;
				isNew = false;
			}
		}
		
		if (isNew) newSeqRuns[newSeqRuns.length] = event.detail.id;
		
		if (event.detail.remaining == 0)
		{
			if (newSeqRuns.length > 0)
			{
				// Get more information about the selected sequencing run
				var url = '../DemuxMerge.servlet?ID='+App.getSessionId();
				url += '&cmd=GetSequencingRunInfo&ids=' + newSeqRuns.join(',');
				Wizard.asyncJsonRequest(url, demux.seqRunInfoLoaded);
			}
			else
			{
				demux.seqRunsOnChange();
			}
		}
	}
	
	demux.seqRunInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var seqRuns = response.sequencingRuns;
		var seqRunList = frm.seqRuns;
		for (var seqNo = 0; seqNo < seqRuns.length; seqNo++)
		{
			var seqRun = seqRuns[seqNo];
			var option = demux.getOptionForSeqRun(seqRun);
			option.selected = true;
			seqRunList[seqRunList.length] = option;
		}
		demux.seqRunsOnChange();
	}

	// Check if all selected sequencing runs have the same pools
	demux.seqRunsOnChange = function()
	{
		seqRunsAreValid = false;
		
		var frm = document.forms['reggie'];
		var numSelected = 0;
		var pools;
		var numDoNotUse = 0;
		var hasPoolMix = false;
		var invalidSelection = null;
		Wizard.setInputStatus('seqRuns');
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected)
			{
				var sr = frm.seqRuns[seqNo].seqRun;
				if (sr.pipeline && sr.pipeline != pipeline.name)
				{
					invalidSelection = Strings.encodeTags(sr.name+' ('+sr.pipeline)+') is not intended for the '+pipeline.name+' pipeline.';
				}
				else if (!sr.DataFilesFolder)
				{
					invalidSelection = Strings.encodeTags(sr.name+' is missing DataFilesFolder information');
				}

				numSelected++;
				if (numSelected == 1)
				{
					pools = sr.flowCell.pools;
					numDoNotUse = sr.flowCell.numDoNotUse;
				}
				else
				{
					if (!demux.compareListOfPools(pools, sr.flowCell.pools))
					{
						hasPoolMix = true;
					}
				}
			}
		}
		if (numSelected == 0)
		{
			Wizard.setInputStatus('seqRuns', 'invalid', 'Select at least one sequencing run');
			return;
		}
		if (invalidSelection) 
		{
			Wizard.setInputStatus('seqRuns', 'invalid', invalidSelection);
			return;
		}

		seqRunsAreValid = true;
		if (hasPoolMix)
		{
			Wizard.setInputStatus('seqRuns', 'warning', 'The selected sequencing runs have different pools');
		}
		else if (numDoNotUse > 0)
		{
			var msg = numDoNotUse == 1 ? '1 library is marked with DoNotUse.' :
				numDoNotUse + ' libraries are marked with DoNotUse.'
			Wizard.setInputStatus('seqRuns', 'warning', msg);
		}
		else
		{
			Wizard.setInputStatus('seqRuns', 'valid');
		}
	}
	
	// Check if two lists of pools are the same or not
	demux.compareListOfPools = function(p1, p2)
	{
		if (p1.length != p2.length) return false;
		for (var i = 0; i < p1.length; i++)
		{
			if (p1[i].id != p2[i].id) return false;
		}
		return true;
	}
	
	demux.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		// ReadString input fields
		var html = '';
		var numDoNotUse = 0;
		var uniqueLibs = {};
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected)
			{
				var seqRun = frm.seqRuns[seqNo].seqRun;
				var fc = seqRun.flowCell;
				for (var libNo = 0; libNo < fc.libraries.length; libNo++)
				{
					var lib = fc.libraries[libNo];
					if (!uniqueLibs[lib.name])
					{
						if (lib.DO_NOT_USE) numDoNotUse++;
						allLibs[allLibs.length] = lib;
						uniqueLibs[lib.name] = true;
					}
				}
				
				var readStringId = 'readString.'+seqNo;
				html += '<tr>';
				html += '<td class="prompt" colspan="2">'+Strings.encodeTags(frm.seqRuns[seqNo].text)+'</td>';
				html += '<td class="status"></td>';
				html += '<td class="help"></td>';
				html += '</tr>';

				html += '<tr>';
				html += '<td class="subprompt">Sequencing cycles</td>';
				html += '<td>'+Strings.encodeTags(seqRun.SequencingCycles)+'</td>';
				html += '<td class="status"></td>';
				html += '<td class="help"></td>';
				html += '</tr>';
				
				html += '<tr>';
				html += '<td class="subprompt">ReadString</td>';
				html += '<td><input type="text" class="required" name="'+readStringId+'" id="'+readStringId+'" value="'+Strings.encodeTags(seqRun.ReadString)+'" style="width: 15em;"></td>';
				html += '<td class="status" id="'+readStringId+'.status"></td>';
				html += '<td class="help"><span id="'+readStringId+'.message"></span></td>';
				html += '</tr>';
				
				html += '<tr>';
				html += '<td class="subprompt">Omit lanes</td>';
				html += '<td>';
				for (var laneNo = 1; laneNo <= fc.numLanes; laneNo++)
				{
					var checked = fc.FailedLanes.indexOf(laneNo) >= 0 ? 'checked' : '';
					html += '<label><input type="checkbox" name="omitLanes.'+seqNo+'" value="'+laneNo+'" ' + checked + '>'+laneNo+'</label> ';
				}
				html += '</td>';
				html += '<td class="status"></td>';
				html += '<td class="help">By default, failed lanes are omitted</td>';
				html += '</tr>';
				
				html += '<tr>';
				html += '<td class="subprompt"></td>';
				html += '<td><div id="check.'+seqNo+'" class="button basicbutton interactable" data-seqrun-id="'+seqRun.id+'"><img src="../images/validate.png" alt="Check data files&amp;hellip;">Check data files&hellip;</div></td>';
				html += '<td class="status"></td>';
				html += '<td class="help"></td>';
				html += '</tr>';
			}
		}
		Doc.element('SequencingRun-section').innerHTML = html;
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected)
			{
				demux.readStringOnBlur(frm['readString.'+seqNo]);
				Events.addEventHandler('readString.'+seqNo, 'blur', demux.readStringOnBlur);
				Buttons.addClickHandler('check.'+seqNo, demux.checkDataFilesOnClick);
			}
		}
		
		if (numDoNotUse > 0)
		{
			Doc.element('numDoNotUseLibs').innerHTML = numDoNotUse == 1 ? '1 library' : numDoNotUse + ' libraries';
			Doc.show('doNotUseOption');
			Wizard.setInputStatus('whichLibs', 'warning', 'By default, libraries marked with DoNotUse are omitted.')
			Forms.checkRadio(frm.whichLibs, 'allExceptDoNotUse');
		}
		
		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('goregister');
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('DEMUX_PROTOCOL', 'demuxProtocol', 'PIPELINE,DEMUX_TYPE', pipeline.name+',Picard');
		Reggie.loadSoftware('DEMUX_SOFTWARE', 'demuxSoftware', 'PIPELINE,DEMUX_TYPE,PARAMETER_SET', pipeline.name+',Picard');
		Reggie.loadProtocols('MERGE_PROTOCOL', 'mergeProtocol', 'PIPELINE,MERGE_TYPE', pipeline.name+',Default');
		Reggie.loadSoftware('MERGE_SOFTWARE', 'mergeSoftware', 'PIPELINE,MERGE_TYPE,PARAMETER_SET', pipeline.name+',Default');

		// Load clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config='+(pipeline.id == 'MIPS' ? 'demux-mips' : 'demux');
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, demux.clustersLoaded);
	}

	demux.showDoNotUseInfo = function(event)
	{
		var libs = allLibs;
		
		var msg = '<table>';
		for (var libNo = 0; libNo < libs.length; libNo++)
		{
			var lib = libs[libNo];
			if (lib.DO_NOT_USE)
			{
				msg += '<tr><td><b>'+Strings.encodeTags(lib.name) + '</b></td><td><span class="itemsubtype">[' + Strings.encodeTags(lib.DO_NOT_USE) + ']</span></td>';
				msg += '<td>'+ Strings.encodeTags(lib.DO_NOT_USE_COMMENT) + '</td></tr>';
			}
		}
		msg += '</table>';
		Forms.showNotification(event.currentTarget, msg, 'donotuseinfo', 'pointer-left');
	}
	
	demux.selectLibsAll = function()
	{
		Wizard.setInputStatus('whichLibs', 'valid');
	}

	demux.selectLibsAllExceptDoNotUse = function()
	{
		Wizard.setInputStatus('whichLibs', 'valid');
	}
	
	demux.selectLibsForDemux = function()
	{
		Wizard.setInputStatus('whichLibs');
		var url = 'select_libs.jsp?ID='+App.getSessionId();
		Dialogs.openPopup(url, 'SelectLibsForDemux', 800, 600);
	}
	
	demux.getAllLibraries = function()
	{
		return allLibs;
	}
	
	demux.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available clusters');
			clusterIsValid = false;
		}
		else
		{
			demux.clusterOnChange();
		}
	}

	demux.checkDataFilesOnClick = function(event)
	{
		var frm = document.forms['reggie'];
		var seqNo = parseInt(event.currentTarget.id.substring(6));
		var url = 'check_data_files.jsp?ID='+App.getSessionId();
		url += '&seqrun='+Data.get(event.currentTarget, 'seqrun-id');
		url += '&readString='+encodeURIComponent(frm['readString.'+seqNo].value);
		
		var omitLanes = [];
		var checkboxes = frm['omitLanes.'+seqNo];
		for (var i = 0; i < checkboxes.length; i++)
		{
			if (checkboxes[i].checked) 
			{
				omitLanes[omitLanes.length] = parseInt(checkboxes[i].value);
			}
		}
		if (omitLanes.length > 0) url += '&omitLanes='+omitLanes.join(',');
		
		Dialogs.openPopup(url, 'CheckDataFiles', 900, 600);
	}

	
	demux.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
		
		var slots = '';
		var demuxConfig = pipeline.id == 'MIPS' ? 'demux-mips' : 'demux';
		if (cluster.config && cluster.config[demuxConfig])
		{
			slots = cluster.config[demuxConfig]['parallel-environment'] || '';
			slots = slots.replace('smp ', '');
		}
		frm.slots.value = slots;
		Events.sendChangeEvent('slots');
	}

	demux.slotsOnChange = function()
	{
		var frm = document.forms['reggie'];
		slotsIsValid = false;
		
		var slots = frm.slots.value;
		if (slots != '' && !slots.match(/^\d+\-?\d*$/))
		{
			Wizard.setInputStatus('slots', 'invalid', 'Invalid slot specification');
			return;
		}
		
		slotsIsValid = true;
		Wizard.setInputStatus('slots', 'valid');
	}
	
	demux.readStringOnBlur = function(eventOrTarget)
	{
		var target = eventOrTarget.currentTarget || eventOrTarget;
		var isValid = false;

		Wizard.setInputStatus(target.id);
		var readString = target.value;
		
		if (!readString.match(/^(\d+[TBMS])+$/))
		{
			Wizard.setInputStatus(target.id, 'invalid', 'Invalid read string');
		}
		else
		{
			isValid = true;
			Wizard.setInputStatus(target.id, 'valid');
		}
		
		Data.set(target, 'valid', isValid ? 1 : 0);
	}

	demux.softwareOnChange = function(event)
	{
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, demux.parameterSetInfoLoaded);
		}
	}
	
	demux.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}

	
	demux.validateStep2 = function(event)
	{
		var frm = document.forms['reggie'];
		if (frm.whichLibs.value == 'selected')
		{
			var numSelected = 0;
			for (var libNo = 0; libNo < allLibs.length; libNo++)
			{
				if (allLibs[libNo].selected) numSelected++;
			}
			if (numSelected == 0)
			{
				Wizard.setInputStatus('whichLibs', 'invalid', 'Please select at least one library');
				event.preventDefault();
				return;
			}
		}
		
		if (!clusterIsValid || !slotsIsValid)
		{
			event.preventDefault();
			return;
		}
			
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected) 
			{
				if (!Wizard.isValid(frm['readString.'+seqNo]))
				{
					event.preventDefault();
					return;
				}
			}
		}
	}
	
	demux.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.demuxSoftware = parseInt(frm.demuxSoftware.value);
		submitInfo.demuxProtocol = parseInt(frm.demuxProtocol.value);
		submitInfo.mergeSoftware = parseInt(frm.mergeSoftware.value);
		submitInfo.mergeProtocol = parseInt(frm.mergeProtocol.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.slots = frm.slots.value;
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
		submitInfo.whichLibs = frm.whichLibs.value;
		if (frm.whichLibs.value == 'selected')
		{
			var ignoreLibs = [];
			for (var libNo = 0; libNo < allLibs.length; libNo++)
			{
				var lib = allLibs[libNo];
				if (!lib.selected) ignoreLibs[ignoreLibs.length] = lib.id;
			}
			submitInfo.ignoreLibs = ignoreLibs;
		}
		
		var sequencingRuns = [];
		submitInfo.sequencingRuns = sequencingRuns;
		
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected) 
			{
				var seqRun = {};
				seqRun.id = frm.seqRuns[seqNo].seqRun.id;
				seqRun.ReadString = frm['readString.'+seqNo].value;
				
				var omitLanes = [];
				var checkboxes = frm['omitLanes.'+seqNo];
				for (var i = 0; i < checkboxes.length; i++)
				{
					if (checkboxes[i].checked) 
					{
						omitLanes[omitLanes.length] = parseInt(checkboxes[i].value);
					}
				}
				seqRun.OmitLanes = omitLanes;
				sequencingRuns[sequencingRuns.length] = seqRun;
			}
		}
		
		var url = '../DemuxMerge.servlet?ID='+App.getSessionId();
		url += '&cmd=StartDemuxAndMerge&pipeline='+encodeURIComponent(Data.get('page-data', 'pipeline'));		
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, demux.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	demux.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return demux;
}();

Doc.onLoad(Demux.initPage);

