var Hisat = function()
{
	var hisat = {};
	var debug = 0;
	
	var JOB_IMAGE = { 'DONE': 'ok.png', 'ERROR': 'error.png'};
	
	var alignedSequences;
	
	// Page initialization
	hisat.initPage = function()
	{
		
		// Step 1

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', hisat.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Hisat2023.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnconfirmedAlignedSequences';
		Wizard.asyncJsonRequest(url, hisat.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	hisat.initializeStep1 = function(response)
	{
		alignedSequences = response.alignedSequences;
		if (alignedSequences.length == 0)
		{
			Wizard.setFatalError('No aligned sequences available for processing.');
			return;
		}
		
		var html = '<table id="alignedSequencesTable">';
		html += '<thead class="bg-filled-100">';
		// Header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="6">Reads (millions)</th>';
		html += '<th class="dottedleft" colspan="3">Genotypes QC</th>'
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="3">Actions</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		
		html += '<tr>';
		html += '<th>Library</th>';
		html += '<th class="dottedleft">Plate</th>';
		html += '<th class="dottedleft">PF_READS</th>';
		html += '<th>PT_READS</th>';
		html += '<th>PM_READS</th>';
		html += '<th>ALIGNED_PAIRS</th>';
		html += '<th class="icon-col"></th>';
		html += '<th>Duplicates</th>';
		html += '<th class="dottedleft">Count</th>';
		html += '<th>HET</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft">Job</th>';
		html += '<th class="dottedleft">Files</th>';
		html += '<th class="dottedleft"><span id="flagrna" data-prefix="flag" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Flag RNA</span></th>';
		html += '<th><span id="realign" data-prefix="realign" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Re-align</span></th>';
		html += '<th><span id="stringtie" data-prefix="stringtie" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">StringTie</span></th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		var numWithError = 0;
		var yellowImg = '<img src="../images/yellow-label.png">';
		for (var alignedNo = 0; alignedNo < alignedSequences.length; alignedNo++)
		{
			var aligned = alignedSequences[alignedNo];

			var lib = aligned.lib;
			lib.stratagene = Reggie.isStratagene(lib.name);
			lib.external = Reggie.isExternal(lib.name);
			var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
			var img = isYellow ? yellowImg : '';

			var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
			
			var merged = aligned.merged;
			var masked = aligned.masked;
			
			var pfReads = merged.PF_READS == null ? '-' : Reggie.formatNumber(merged.PF_READS/1000000, null, 2)
			var ptReads = merged.PT_READS == null ? '-' : Reggie.formatNumber(merged.PT_READS/1000000, null, 2)
			var maskedReads = masked.PM_READS == null ? '-' : Reggie.formatNumber(masked.PM_READS/1000000, null, 2)
			var alignedReads = aligned.ALIGNED_PAIRS == null ? '-' : Reggie.formatNumber(aligned.ALIGNED_PAIRS/1000000, null, 2);
			var ptPercent = merged.PF_READS && merged.PT_READS ? ' (' + Math.round(100*merged.PT_READS/merged.PF_READS) + '%)' : '';
			var alignedPercent = aligned.ALIGNED_PAIRS && merged.PF_READS ? ' (' + Math.round(100*aligned.ALIGNED_PAIRS/merged.PF_READS) + '%)' : '';
			var percentDuplicates = aligned.FRACTION_DUPLICATION == null ? '-' : Math.round(100*aligned.FRACTION_DUPLICATION)+'%';
			var genotypeCount = aligned.QC_GENOTYPE_COUNT || '-';
			var hetPct = aligned.QC_GENOTYPE_HET_PCT == null ? '' : Math.round(aligned.QC_GENOTYPE_HET_PCT)+'%';
			var job = aligned.job;
			var limits = job.limits;
			
			var flag = null;
			var isError = job.status == 'ERROR';
			var stringTieChecked = aligned.ALIGNED_PAIRS != null && aligned.ALIGNED_PAIRS > limits.minAlignedPairsStop;
			
			html += '<tr class="highlight ' + (alignedNo % 4 < 2 ? 'evenrow' : 'oddrow') + (isYellow ? ' yellow-specimen' : '') +  '">';
			html += '<td class="prompt if-yellow">'+img+aligned.name+'</td>';
			html += '<td class="dottedleft">'+(libPlate ? Strings.encodeTags(libPlate.name) : '-')+'</td>';
			html += '<td class="dottedleft">'+pfReads+'</td>';
			html += '<td>'+ptReads+ptPercent+'</td>';
			html += '<td>'+maskedReads+'</td>';
			html += '<td>'+alignedReads+alignedPercent+'</td>';
			html += '<td class="icon-col">';
			if (aligned.ALIGNED_PAIRS != null && aligned.ALIGNED_PAIRS < limits.minAlignedPairsFlag)
			{
				html += '<img src="../images/warning.png" title="Less than '+Reggie.formatCount(limits.minAlignedPairsFlag) + ' aligned pairs">';
				flag = 'AlignFailed';
			}
			html += '</td>';

			html += '<td>'+percentDuplicates+'</td>';
			html += '<td class="dottedleft">'+genotypeCount+'</td>';
			html += '<td>'+hetPct;
			if (aligned.QC_GENOTYPE_HET_PCT == null)
			{
				html += '<img src="../images/warning.png" title="Genotype QC failed">';
				isError = true;
			}
			else if (aligned.QC_GENOTYPE_HET_PCT > limits.maxHetPct)
			{
				html += '<img src="../images/warning.png" title="More than '+Reggie.formatCount(limits.maxHetPct) + '% HET may indicate contamination">';
				if (flag == null) flag = 'HighHET';
			}
			html += '</td>';
			html += '<td class="icon-col">';
			if  (aligned.qcVcf)
			{
				html += '<span class="link vcf-link" data-file-id="'+aligned.qcVcf.id+'" data-item-id="'+aligned.id+'" title="View VCF statistics"><img src="../images/vcf_file.png"></span>';
			}
			html += '</td>';
			html += '<td class="dottedleft">';
			html += '<span class="link item-link" data-item-type="JOB" data-item-id="'+job.id+'" title="'+Strings.encodeTags(job.statusMessage)+'"><img src="../images/'+JOB_IMAGE[job.status]+'"></span>';
			html += '</td>';
			if (aligned.DataFilesFolder && job.server)
			{
				html += '<td class="dottedleft">';
				html += '<span class="link file-link" data-server="'+Strings.encodeTags(job.server)+'" data-folder="'+Strings.encodeTags(aligned.DataFilesFolder)+'" data-item-id="'+aligned.id+'" title="View generated files"><img src="../images/remote_folder.png"></span>';
				html += '</td>';
			}
			else
			{
				html += '<td class="dottedleft">-</td>';
			}
			
			var realignChecked = isError ? ' checked' : '';
			var stringTieDisabled = isError ? ' disabled' : '';
			if (isError) numWithError++;
			if (lib.stratagene || lib.external)
			{
				html += '<td class="dottedleft">-</td>';
			}
			else
			{
				html += '<td class="dottedleft"><input type="checkbox" name="flag.'+aligned.id+'"'+(flag != null ? ' checked value="'+flag+'"':'')+'></td>';
			}
			
			html += '<td><input type="checkbox" name="realign.'+aligned.id+'"'+realignChecked+'></td>';
			html += '<td><input type="checkbox" name="stringtie.'+aligned.id+'"'+stringTieDisabled+(stringTieChecked ? ' checked':'')+'></td>';
			html += '<td class="dottedleft comment"><input type="text" name="comment.'+aligned.id+'"></td>';
			html += '</tr>';

		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('alignedSequences').innerHTML = html;
		if (numWithError > 0) Doc.show('delete-items', 'table');
		
		Events.addEventHandler('flagrna', 'click', hisat.toggleSelection);
		Events.addEventHandler('realign', 'click', hisat.toggleSelection);
		Events.addEventHandler('stringtie', 'click', hisat.toggleSelection);
		
		// Add click handler to items
		var clickableItems = document.getElementsByClassName('item-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		
		// Add click handler to file links
		var clickableItems = document.getElementsByClassName('file-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', hisat.fileLinkOnClick);
		}

		// Add click handler to VCF files
		var clickableItems = document.getElementsByClassName('vcf-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', hisat.vcfLinkOnClick);
		}

		Doc.show('step-1');
		Doc.show('goregister');
	}
	
	hisat.fileLinkOnClick = function(event)
	{
		var folder = Data.get(event.currentTarget, 'folder');
		var server = Data.get(event.currentTarget, 'server');
		var itemId = Data.get(event.currentTarget, 'item-id');
		server = server.replace(/\s*\[.*\]/, '');
		var url = 'view_remote_files.jsp?ID=' + App.getSessionId();
		url += '&clusterId='+server;
		url += '&path='+encodeURIComponent(folder);
		url += '&itemType=DERIVEDBIOASSAY&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewProjectFiles', 750, 500);
	}
	
	hisat.vcfLinkOnClick = function(event)
	{
		var fileId = Data.int(event.currentTarget, 'file-id');
		var itemId = Data.get(event.currentTarget, 'item-id');

		var url = 'view_genotypes.jsp?ID=' + App.getSessionId();
		url += '&fileId='+fileId;
		url += '&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewVcf', 750, 500);
	}
	
	hisat.toggleSelection = function(event)
	{
		var prefix = Data.get(event.currentTarget, 'prefix');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		
		var frm = document.forms['reggie'];
		for (var alignedNo = 0; alignedNo < alignedSequences.length; alignedNo++)
		{
			var aligned = alignedSequences[alignedNo];
			var chk = frm[prefix+'.'+aligned.id];
			if (chk && !chk.disabled)
			{
				chk.checked = specialKey ? false : !chk.checked;
			}
		}
	}
	
	hisat.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.deleteItemsCreatedByFailedJobs = frm.deleteItemsCreatedByFailedJobs.checked;
		
		var all = [];
		submitInfo.alignedSequences = all;
		for (var alignedNo = 0; alignedNo < alignedSequences.length; alignedNo++)
		{
			var aligned = alignedSequences[alignedNo];
			var tmp = {};
			tmp.id = aligned.id;
			
			tmp.flag = frm['flag.'+aligned.id] && frm['flag.'+aligned.id].checked ? frm['flag.'+aligned.id].value : null;
			tmp.realign = frm['realign.'+aligned.id].checked;
			tmp.stringtie = frm['stringtie.'+aligned.id].checked;
			tmp.comment = frm['comment.'+aligned.id].value;
			
			all[all.length] = tmp;
		}
		
		var url = '../Hisat2023.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterHisat2023Alignment';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, hisat.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	hisat.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	return hisat;
}();

Doc.onLoad(Hisat.initPage);

