var Fastq = function()
{
	var fastq = {};
	var debug = 0;
	
	var ADAPTER_SIZE = 59;
	var JOB_IMAGE = { 'DONE': 'ok.png', 'ERROR': 'error.png'};
	
	var mergedSequences;
	
	// Page initialization
	fastq.initPage = function()
	{
		
		// Step 1

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', fastq.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Fastq.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnconfirmedImportedSequences';
		Wizard.asyncJsonRequest(url, fastq.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	fastq.initializeStep1 = function(response)
	{
		mergedSequences = response.mergedSequences;
		if (mergedSequences.length == 0)
		{
			Wizard.setFatalError('No merged sequences available for processing.');
			return;
		}
		
		var html = '<table id="mergedSequencesTable">';
		html += '<thead class="bg-filled-100">';
		// Header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="5">Reads (Millions)</th>';
		html += '<th class="dottedleft" colspan="4">Fragment size</th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="5">Actions</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		html += '<tr>';
		html += '<th>Library</th>';
		html += '<th class="dottedleft">Plate</th>';
		html += '<th class="dottedleft">READS</th>';
		html += '<th>PF_READS</th>';
		html += '<th>ADAPTER_READS</th>';
		html += '<th>PT_READS</th>';
		html += '<th class="warning-col"></th>';
		html += '<th class="dottedleft">Size*¹</th>';
		html += '<th>Avg²</th>';
		html += '<th>Stdev²</th>';
		html += '<th class="warning-col"></th>';
		html += '<th class="dottedleft">Job</th>';
		html += '<th class="dottedleft">Files</th>';
		html += '<th class="dottedleft"><span id="flagrna" data-prefix="flag" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Flag RNA</span></th>';
		html += '<th><span id="reimport" data-prefix="reimport" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Re-import</span></th>';
		html += '<th><span id="legacy" data-prefix="legacy" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Legacy</span></th>';
		html += '<th><span id="hisat" data-prefix="hisat" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Hisat</span></th>';
		html += '<th><span id="hisat2023" data-prefix="hisat2023" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Hisat/2023</span></th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		
		var yellowImg = '<img src="../images/yellow-label.png">';
		for (var mergedNo = 0; mergedNo < mergedSequences.length; mergedNo++)
		{
			var merged = mergedSequences[mergedNo];
			var lib = merged.lib;
			var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
			var job = merged.job;
			var minPtReads = job.limits.minPtReads;

			var isError = job.status == 'ERROR';
			var flagChecked = false;
			var alignChecked = !isError;
			
			merged.stratagene = Reggie.isStratagene(merged.name);
			merged.external = Reggie.isExternal(merged.name);
			var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
			var img = isYellow ? yellowImg : '';

			var reads = merged.READS == null ? '-' : Reggie.formatNumber(merged.READS/1000000, null, 2);
			var pfReads = merged.PF_READS == null ? '-' : Reggie.formatNumber(merged.PF_READS/1000000, null, 2);
			var ptReads = merged.PT_READS == null ? '-' : Reggie.formatNumber(merged.PT_READS/1000000, null, 2);
			var ptPercent = merged.PF_READS > 0 && merged.PT_READS != null ? Math.round(100*merged.PT_READS/merged.PF_READS) : Number.NaN;
			var adapterReads = merged.ADAPTER_READS == null ? '-' : Reggie.formatNumber(merged.ADAPTER_READS/1000000, null, 2);
			var adapterPercent = merged.PF_READS > 0 && merged.ADAPTER_READS != null ? Math.round(100*merged.ADAPTER_READS/merged.PF_READS) : Number.NaN;
			
			html += '<tr class="highlight ' + (isYellow ? 'yellow-specimen' : '') + '">';
			html += '<td class="prompt if-yellow">'+img+Strings.encodeTags(merged.name)+'</td>';
			html += '<td class="dottedleft">'+(libPlate ? Strings.encodeTags(libPlate.name) : '-')+'</td>';
			// READS
			html += '<td class="dottedleft">'+reads+'</td>';
			html += '<td>'+pfReads+'</td>';
			html += '<td>'+adapterReads+(isNaN(adapterPercent) ? '' : ' ('+adapterPercent+'%)') +'</td>';
			html += '<td>'+ptReads+(isNaN(ptPercent) ? '' : ' ('+ptPercent+'%)') + '</td>';
			html += '<td class="warning-col">';
			if (merged.PT_READS == null || merged.PT_READS < minPtReads)
			{
				html += '<img src="../images/warning.png" title="Less than '+Reggie.formatCount(minPtReads) + ' passed Trimmomatic">';
				if (!isError)
				{
					flagChecked = true;
					alignChecked = false;
				}
			}
			html += '</td>';

			// Fragment sizes
			var lib_size = Math.round(lib.CA_Size || lib.library_size_est) - ADAPTER_SIZE * 2;
			html += '<td class="dottedleft">'+lib_size+'</td>';
			html += '<td>'+(merged.FragmentSizeAvg || '-')+'</td>';
			html += '<td>'+(merged.FragmentSizeStdev || '-')+'</td>';
			html += '<td class="warning-col">';
			if (merged.FragmentSizeAvg != null && merged.FragmentSizeStdev != null)
			{
				var delta = lib_size - merged.FragmentSizeAvg;
				if (Math.abs(delta) > merged.FragmentSizeStdev)
				{
					html += '<img src="../images/warning.png" title="Fragment size differ from CA_Size: ' + delta+'">';
				}
			}
			html += '</td>';
			
			// Job/Files
			html += '<td class="dottedleft">';
			html += '<span class="link item-link" data-item-type="JOB" data-item-id="'+job.id+'" title="'+Strings.encodeTags(job.statusMessage)+'"><img src="../images/'+JOB_IMAGE[job.status]+'"></span>';
			html += '</td>';
			if (merged.DataFilesFolder && job.server)
			{
				html += '<td class="dottedleft">';
				html += '<span class="link file-link" data-server="'+Strings.encodeTags(job.server)+'" data-folder="'+Strings.encodeTags(merged.DataFilesFolder)+'" data-item-id="'+merged.id+'" title="View generated files"><img src="../images/remote_folder.png"></span>';
				html += '</td>';
			}
			else
			{
				html += '<td class="dottedleft">-</td>';
			}

			// Actions
			// Do not allow flagging external or stratagene
			if (merged.stratagene || merged.external)
			{
				html += '<td class="dottedleft">-</td>';
			}
			else
			{
				html += '<td class="dottedleft"><input type="checkbox" name="flag.'+merged.id+'"'+(flagChecked ? ' checked':'')+'></td>';
			}
			
			var alignDisabled = isError ? ' disabled' : '';
			html += '<td><input type="checkbox" name="reimport.'+merged.id+'"'+(isError ? ' checked':'')+'></td>';
			html += '<td><input type="checkbox" name="legacy.'+merged.id+'" '+(alignChecked ? ' checked':'')+alignDisabled+'></td>';
			html += '<td><input type="checkbox" name="hisat.'+merged.id+'" '+(alignChecked ? ' checked':'')+alignDisabled+'></td>';
			html += '<td><input type="checkbox" name="hisat2023.'+merged.id+'" '+(alignChecked ? ' checked':'')+alignDisabled+'></td>';
			html += '<td class="dottedleft comment"><input type="text" name="comment.'+merged.id+'"></td>';
			html += '</tr>';
		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('mergedSequences').innerHTML = html;
	
		Events.addEventHandler('flagrna', 'click', fastq.toggleSelection);
		Events.addEventHandler('reimport', 'click', fastq.toggleSelection);
		Events.addEventHandler('legacy', 'click', fastq.toggleSelection);
		Events.addEventHandler('hisat', 'click', fastq.toggleSelection);
		Events.addEventHandler('hisat2023', 'click', fastq.toggleSelection);

		// Add click handler to items
		var clickableItems = document.getElementsByClassName('item-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		
		// Add click handler to file links
		var clickableItems = document.getElementsByClassName('file-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', fastq.fileLinkOnClick);
		}

		Doc.show('step-1');
		Doc.show('goregister');
	}
	
	fastq.fileLinkOnClick = function(event)
	{
		var folder = Data.get(event.currentTarget, 'folder');
		var server = Data.get(event.currentTarget, 'server');
		var itemId = Data.get(event.currentTarget, 'item-id');
		server = server.replace(/\s*\[.*\]/, '');
		var url = 'view_remote_files.jsp?ID=' + App.getSessionId();
		url += '&clusterId='+server;
		url += '&path='+encodeURIComponent(folder);
		url += '&itemType=DERIVEDBIOASSAY&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewProjectFiles', 750, 500);
	}
	
	fastq.toggleSelection = function(event)
	{
		var prefix = Data.get(event.currentTarget, 'prefix');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		
		var frm = document.forms['reggie'];
		for (var mergedNo = 0; mergedNo < mergedSequences.length; mergedNo++)
		{
			var merged = mergedSequences[mergedNo];
			var chk = frm[prefix+'.'+merged.id];
			if (chk && !chk.disabled)
			{
				chk.checked = specialKey ? false : !chk.checked;
			}
		}
	}

	fastq.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.deleteItemsCreatedByFailedJobs = frm.deleteItemsCreatedByFailedJobs.checked;
		
		var all = [];
		submitInfo.mergedSequences = all;
		for (var mergedNo = 0; mergedNo < mergedSequences.length; mergedNo++)
		{
			var merged = mergedSequences[mergedNo];
			var tmp = {};
			tmp.id = merged.id;
			
			tmp.flag = frm['flag.'+merged.id] && frm['flag.'+merged.id].checked;
			tmp.reimport = frm['reimport.'+merged.id].checked;
			tmp.hisatAlign = frm['hisat.'+merged.id].checked;
			tmp.hisat2023Align = frm['hisat2023.'+merged.id].checked;
			tmp.legacyAlign = frm['legacy.'+merged.id].checked;
			tmp.comment = frm['comment.'+merged.id].value;
			
			all[all.length] = tmp;
		}
		
		var url = '../Fastq.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterFastqImport';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, fastq.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	fastq.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	return fastq;
}();

Doc.onLoad(Fastq.initPage);

