var Fastq = function()
{
	var fastq = {};
	var debug = 0;
	var selectionIsValid = false;
	var clusterIsValid = false;
	
	// Page initialization
	fastq.initPage = function()
	{
		
		// Step 1
		Events.addEventHandler('demuxedSequences', 'change', fastq.demuxedSequencesOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', fastq.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', fastq.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', fastq.validateStep2);
		Events.addEventHandler('clusters', 'change', fastq.clusterOnChange);
		Events.addEventHandler('mergeSoftware', 'change', fastq.softwareOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', fastq.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Fastq.servlet?ID='+App.getSessionId();
		url += '&cmd=GetDemuxedSequencesForImport';
		Wizard.asyncJsonRequest(url, fastq.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	fastq.initializeStep1 = function(response)
	{
		var demuxedSequences = response.demuxedSequences;
		var frm = document.forms['reggie'];	
		if (demuxedSequences.length == 0)
		{
			Wizard.setFatalError('No demuxed sequences available for importing.');
			return;
		}

		var firstPlate = null;
		for (var msNo=0; msNo < demuxedSequences.length; msNo++)
		{
			var ms = demuxedSequences[msNo];
			var libPlate = ms.lib.bioWell ? ms.lib.bioWell.bioPlate : null;
			if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
			var option = fastq.createListOption(msNo+1, ms, ms.DO_NOT_USE == null && libPlate != null && libPlate.name==firstPlate)
			frm.demuxedSequences.options[frm.demuxedSequences.length] = option;
		}
		Events.sendChangeEvent('demuxedSequences');
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.demuxedSequences.focus();
	}
	
	fastq.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}
	
	fastq.createListOption = function(index, demuxItem, selected)
	{
		var lib = demuxItem.lib;
		var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
		var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
		
		var tooltip = null;
		var name = (index) + ': ';
		if (libPlate) name += libPlate.name + ' - ';
		name += demuxItem.name;
		
		if (demuxItem.DO_NOT_USE)
		{
			name += ' [DoNotUse]';
			tooltip = 'DoNotUse-'+Strings.encodeTags(demuxItem.DO_NOT_USE+': '+demuxItem.DO_NOT_USE_COMMENT);
		}
		else if (demuxItem.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		var option = new Option(name, demuxItem.id, false, selected);
		if (isYellow) option.className = 'yellow';
		if (tooltip) option.title = tooltip;
		option.demuxedSequences = demuxItem;
		return option;
	}
	
	fastq.demuxedSequencesOnChange = function()
	{
		var frm = document.forms['reggie'];
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidSelection = null;
		
		Wizard.setInputStatus('demuxedSequences');
		Wizard.hideGoNextConfirmation();
		for (var msNo = 0; msNo < frm.demuxedSequences.length; msNo++)
		{
			if (frm.demuxedSequences[msNo].selected) 
			{
				numSelected++;
				var ms = frm.demuxedSequences[msNo].demuxedSequences;
				if (ms.DO_NOT_USE) numDoNotUse++;
				if (ms.pipeline && ms.pipeline != 'RNAseq')
				{
					invalidSelection = Strings.encodeTags(ms.name+' ('+ms.pipeline)+') is not intended for the RNAseq pipeline.';
				}
				else if (ms.rawFastq.length == 0)
				{
					invalidSelection = Strings.encodeTags(ms.name+' is missing RawFASTQ information');
				}
				else if (!ms.DataFilesFolder)
				{
					invalidSelection = Strings.encodeTags(ms.name+' is missing DataFilesFolder information');
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('demuxedSequences', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidSelection) 
		{
			Wizard.setInputStatus('demuxedSequences', 'invalid', invalidSelection);
			return;
		}
		selectionIsValid = true;
		
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('demuxedSequences', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('demuxedSequences', 'valid');
		}
	}
	
	// --- Step 2 -----------------------------------
	fastq.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('MERGE_PROTOCOL', 'mergeProtocol', 'PIPELINE,MERGE_TYPE', 'RNAseq,Import');
		Reggie.loadSoftware('MERGE_SOFTWARE', 'mergeSoftware', 'PIPELINE,MERGE_TYPE,PARAMETER_SET', 'RNAseq,Import');
		
		// Load clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config=demux/execute-import';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, fastq.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	fastq.validateStep2 = function(event)
	{
		if (!clusterIsValid) event.preventDefault();
	}
	
	fastq.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available clusters');
			clusterIsValid = false;
		}
		else
		{
			fastq.clusterOnChange();
		}
	}
	
	fastq.softwareOnChange = function(event)
	{
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, fastq.parameterSetInfoLoaded);
		}
	}
	
	fastq.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	fastq.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	

	fastq.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.mergeSoftware = parseInt(frm.mergeSoftware.value);
		submitInfo.mergeProtocol = parseInt(frm.mergeProtocol.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
		
		var demuxedSequences = [];
		submitInfo.demuxedSequences = demuxedSequences;
			
		for (var msNo = 0; msNo < frm.demuxedSequences.length; msNo++)
		{
			if (frm.demuxedSequences[msNo].selected) 
			{
				var ms = {};
				ms.id = frm.demuxedSequences[msNo].demuxedSequences.id;
				demuxedSequences[demuxedSequences.length] = ms;
			}
		}

		var url = '../Fastq.servlet?ID='+App.getSessionId();
		url += '&cmd=ImportFastq';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, fastq.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	fastq.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return fastq;
}();

Doc.onLoad(Fastq.initPage);

