var VCall = function()
{
	var vcall = {};
	var debug = 0;
	var selectionIsValid = false;
	var subtypeAlignedSequences = null;
	var annotationTypePipeline = null;
	var fileTypeBAM = null;
	
	var clusterIsValid = false;
	var manuallySelected = [];
	
	// Page initialization
	vcall.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectAlignedSequences', vcall.selectAlignedSequences);
		Events.addEventHandler('alignedSequences', 'base-selected', vcall.setAlignedSequenceCallback);
		Events.addEventHandler('alignedSequences', 'change', vcall.alignedSequencesOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', vcall.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', vcall.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', vcall.validateStep2);
		Events.addEventHandler('clusters', 'change', vcall.clusterOnChange);
		Events.addEventHandler('vcallSoftware', 'change', vcall.softwareOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', vcall.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../VariantCalling.servlet?ID='+App.getSessionId();
		url += '&cmd=GetAlignedSequencesForVariantCalling';
		Wizard.asyncJsonRequest(url, vcall.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	vcall.initializeStep1 = function(response)
	{
		var alignedSequences = response.alignedSequences;
		var frm = document.forms['reggie'];	

		if (alignedSequences != null && alignedSequences.length > 0)
		{
			var firstPlate = null;
			for (var asNo=0; asNo < alignedSequences.length; asNo++)
			{
				var as = alignedSequences[asNo];
				var libPlate = as.lib.bioWell ? as.lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				var option = vcall.createListOption(asNo+1, as, as.DO_NOT_USE == null && libPlate != null && libPlate.name==firstPlate)
				frm.alignedSequences.options[frm.alignedSequences.length] = option;
			}
			Events.sendChangeEvent('alignedSequences');
		}
		else
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', 'No new sequences available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.alignedSequences.focus();
	}
	
	vcall.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}
	
	vcall.selectAlignedSequences = function()
	{
		var frm = document.forms['reggie'];
		if (frm.alignedSequences.disabled) return;
		
		if (subtypeAlignedSequences == null)
		{
			subtypeAlignedSequences = Reggie.getSubtypeInfo('ALIGNED_SEQUENCES');
		}
		if (fileTypeBAM == null)
		{
			fileTypeBAM = Reggie.getDataFileTypeInfo('BAM');
		}
		if (annotationTypePipeline == null)
		{
			annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		}
		
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		// Alignments with Hisat gives us 'AlignedSequences' items with 'alignment.bam'
		url += '&tmpfilter:INT:itemSubtype='+subtypeAlignedSequences.id;
		url += '&tmpfilter:STRING:'+encodeURIComponent('¤'+fileTypeBAM.id+'(file.name)')+'=alignment.bam';
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'='+encodeURIComponent('RNAseq/Hisat/StringTie');

		Dialogs.selectItem('DERIVEDBIOASSAY', 'alignedSequences', 1, url);
	}

	vcall.setAlignedSequenceCallback = function(event)
	{
		var ms = event.detail;
		
		var opt = Reggie.getListOption('alignedSequences', ms.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = ms.id;
		}
		
		if (ms.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../VariantCalling.servlet?ID='+App.getSessionId();
			url += '&cmd=GetAlignedSequencesForVariantCalling';
			url += '&items='+manuallySelected.join(',');
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, vcall.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('alignedSequences');
		}
	}

	vcall.manuallySelected = function(response)
	{
		var alignedSequences = response.alignedSequences;
		var frm = document.forms['reggie'];	

		if (alignedSequences != null && alignedSequences.length > 0)
		{
			var offset = frm.alignedSequences.length+1;
			for (var asNo=0; asNo < alignedSequences.length; asNo++)
			{
				var as = alignedSequences[asNo];
				var option = vcall.createListOption(asNo+offset, as, as.DO_NOT_USE == null);
				frm.alignedSequences.options[frm.alignedSequences.length] = option;
			}
			Events.sendChangeEvent('alignedSequences');
		}
	}

	vcall.createListOption = function(index, alignedItem, selected)
	{
		var lib = alignedItem.lib;
		var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
		var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
		
		var tooltip = null;
		var name = (index) + ': ';
		if (libPlate) name += libPlate.name + ' - ';
		name += alignedItem.name;
		
		var tags = [];
		if (alignedItem.DO_NOT_USE)
		{
			tags[tags.length] = 'DoNotUse';
			tooltip = 'DoNotUse-'+Strings.encodeTags(alignedItem.DO_NOT_USE+': '+alignedItem.DO_NOT_USE_COMMENT);
		}
		if (alignedItem.AutoProcess == 'ReProcess')
		{
			tags[tags.length] = 'R';
		}
		if (alignedItem.rawVariants)
		{
			tags[tags.length] = 'V';
		}
		if (tags.length) name += ' [' + tags.join('; ') + ']';
		
		var option = new Option(name, alignedItem.id, false, selected);
		if (isYellow) option.className = 'yellow';
		if (tooltip) option.title = tooltip;
		option.alignedSequences = alignedItem;
		return option;
	}
	
	vcall.alignedSequencesOnChange = function()
	{
		var frm = document.forms['reggie'];
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;
		Wizard.setInputStatus('alignedSequences');
		Wizard.hideGoNextConfirmation();
		
		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) 
			{
				numSelected++;
				var as = frm.alignedSequences[asNo].alignedSequences;
				if (as.DO_NOT_USE) numDoNotUse++;
				if (as.pipeline && as.pipeline != 'RNAseq/Hisat/StringTie')
				{
					invalidPipeline = Strings.encodeTags(as.name+' ('+as.pipeline)+') is not intended for the RNAseq/Hisat/StringTie pipeline.';
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('alignedSequences', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('alignedSequences', 'valid');
		}
	}

	// --- Step 2 -----------------------------------
	vcall.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadSoftware('VARIANT_CALLING_SOFTWARE', 'vcallSoftware', 'VARIANT_CALL_TYPE,PARAMETER_SET', 'VariantCall');
		
		// Load clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config=variant-call';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, vcall.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	vcall.validateStep2 = function(event)
	{
		if (!clusterIsValid) event.preventDefault();
	}
	
	vcall.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available clusters');
			clusterIsValid = false;
		}
		else
		{
			vcall.clusterOnChange();
		}
	}
	
	vcall.softwareOnChange = function(event)
	{
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, vcall.parameterSetInfoLoaded);
		}
	}
	
	vcall.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	vcall.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	vcall.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.skipRaw = frm.skipRaw.checked;
		submitInfo.rawOnly = frm['callType.rawOnly'].checked;
		submitInfo.vcallSoftware = parseInt(frm.vcallSoftware.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
			
		var alignedSequences = [];
		submitInfo.alignedSequences = alignedSequences;
			
		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) 
			{
				var as = {};
				as.id = frm.alignedSequences[asNo].alignedSequences.id;
				alignedSequences[alignedSequences.length] = as;
			}
		}

		var url = '../VariantCalling.servlet?ID='+App.getSessionId();
		url += '&cmd=StartVariantCalling';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, vcall.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	vcall.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return vcall;
}();

Doc.onLoad(VCall.initPage);

