var CreatePool = function()
{
	var createpool = {};
	var debug = 0;

	var subtypeLibrary;
	var annotationTypePipeline;
	
	var selectedLibraries = [];
	var newLibraries;
	
	var LIMIT_FOR_AUTO_EXCLUDE = 0.25;
	var LIMIT_FOR_EXTRA_LARGE_MIX;
	var MAX_TARGET_VOLUME = 20;
	var MIN_TARGET_VOLUME = 2;
	var LIBRARY_FRAC_ADPT_LIMIT = 10; // percent

	var targetVolumePerLibIsValid = true;
	var lastTargetVolume = null;
	var lastComment = '';
	
	// Page initialization
	createpool.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', createpool.validateStep1);
		Events.addEventHandler('target_molarity', 'change', createpool.updatePoolData);
		Events.addEventHandler('target_volume', 'change', createpool.targetVolumeOnChange);
		Events.doOnEnter('target_volume', createpool.targetVolumeOnChange);
		Events.addEventHandler('target_volume', 'keypress', Events.numberOnly);
		
		Events.addEventHandler('mixing_strategy_dynamic', 'change', createpool.updatePoolData);
		Events.addEventHandler('mixing_strategy_fixed', 'change', createpool.updatePoolData);
		
		Buttons.addClickHandler('btnSelectLibraries', createpool.selectLibraries);
		Events.addEventHandler('lib-table', 'base-selected', createpool.librarySelected);
		Buttons.addClickHandler('btnRemove', createpool.removeSelected);
		Buttons.addClickHandler('btnTargetVolume', createpool.setTargetVolume);
		Buttons.addClickHandler('btnComment', createpool.commentChecked);
		
		Events.addEventHandler('checkAll', 'click', createpool.checkAll);
		
		// Navigation
		Buttons.addClickHandler('gocancel', createpool.cancelWizard);
		Buttons.addClickHandler('gorestart', createpool.cancelWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', createpool.submit);

		var url = '../Pool.servlet?ID='+App.getSessionId();
		url += '&cmd=GetNextAutoGeneratedPoolNames&numNames=1';
		Wizard.showLoadingAnimation('Loading pool information...');
		Wizard.asyncJsonRequest(url, createpool.poolInfoLoaded);
	}
	
	createpool.poolInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];

		// Load some default pooling options and the auto-generated name for the next pool
		var poolInfo = response.poolInfo;
		LIMIT_FOR_EXTRA_LARGE_MIX = poolInfo.limitForExtraLargeMix;
		Doc.element('pool-name').innerHTML = Strings.encodeTags(response.names[0]);
		
		frm.target_volume.value = poolInfo.targetVolumePerLib;
		frm.poolName.value = response.names[0];
		
		Doc.show('step-1');
		Doc.show('gocancel');
		Doc.show('goregister');
		Wizard.setNoConfirmOnFirstStep(false);
	}
	
	createpool.cancelWizard = function()
	{
		location.href = 'create_pools.jsp?ID='+App.getSessionId();
	}

	createpool.selectLibraries = function()
	{
		if (subtypeLibrary == null) subtypeLibrary = Reggie.getSubtypeInfo('LIBRARY');
		if (annotationTypePipeline == null) annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeLibrary.id;
		url += '&tmpfilter:DATE:creationEvent.eventDate='+encodeURIComponent('<>');
		url += '&tmpfilter:STRING:bioWell.bioPlate.name='+encodeURIComponent('<>');;
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'=RNAseq';
		
		var exclude = [];
		for (var libNo = 0; libNo < selectedLibraries.length; libNo++)
		{
			exclude[exclude.length] = selectedLibraries[libNo].id;
		}
		if (exclude.length > 0) url += '&exclude='+exclude.join(',');

		newLibraries = [];
		Dialogs.selectItem('EXTRACT', 'lib-table', 1, url);
	}

	createpool.librarySelected = function(event)
	{
		var id = event.detail.id;

		// Check if this library has already been selected
		var isNew = true;
		for (var i = 0; i < selectedLibraries.length; i++)
		{
			if (selectedLibraries[i].id == id) 
			{
				isNew = false;
				break;
			}
		}
		if (isNew) newLibraries[newLibraries.length] = id;
		
		if (event.detail.remaining == 0 && newLibraries.length > 0)
		{
			// Get more information about the selected library
			var url = '../Pool.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLibraryInfo&libraries=' + newLibraries.join(',');
			Wizard.asyncJsonRequest(url, createpool.libraryInfoLoaded);
		}
	}
	
	createpool.removeSelected = function()
	{
		var frm = document.forms['reggie'];
		var tbody = Doc.element('lib-table-rows');
		var numRemoved = 0;
		for (var libNo = selectedLibraries.length-1; libNo >= 0; libNo--)
		{
			var lib = selectedLibraries[libNo];
			if (frm['check.'+lib.id].checked)
			{
				var tr = document.getElementById('lib.'+lib.id);
				tbody.removeChild(tr);
				selectedLibraries.splice(libNo, 1);
				numRemoved++;
			}
		}
		
		if (numRemoved > 0)
		{
			createpool.updatePoolData();
		}
		
		if (selectedLibraries.length == 0)
		{	
			Doc.show('not-selected');
			Doc.hide('lib-table');
		}
	}
	
	createpool.checkAll = function()
	{
		var frm = document.forms['reggie'];
		var check = null;
		for (var i = 0; i < frm.elements.length; i++)
		{
			var e = frm.elements[i];
			if (e.type == 'checkbox')
			{
				if (check == null) check = !e.checked;
				e.checked = check;
			}
		}
	}
	
	createpool.libraryInfoLoaded = function(response)
	{
		var libraries = response.libraries;
		for (var libNo = 0; libNo < libraries.length; libNo++)
		{
			var lib = libraries[libNo];
			selectedLibraries[selectedLibraries.length] = lib;
			if (lib.pipeline == 'RNAseq') 
			{
				// We do not make this check if the pipeline is not RNAseq since we will catch that as an error in updatePoolData()
				lib.excludeFromPool = lib.molarity > 0 && lib.remainingQuantity > 0 ? false : true;
			}
			createpool.addLibraryToTable(lib);
		}
		
		createpool.updatePoolData();
		
		Doc.hide('not-selected');
		Doc.show('lib-table');
	}

	createpool.addLibraryToTable = function(lib)
	{
		var tr = document.createElement('tr');
		tr.id = 'lib.'+lib.id;
		var className = 'highlight';
		if (lib.excludeFromPool) className += ' excluded';
		createpool.addColumn(tr, 'check', '<input type="checkbox" name="check.'+lib.id+'" value="'+lib.id+'">');
		createpool.addColumn(tr, 'name', Strings.encodeTags(lib.name));
		createpool.addColumn(tr, 'barcode', lib.barcode ? Strings.encodeTags(lib.barcode.name) : '-');
		createpool.addColumn(tr, 'location', lib.bioWell ? Strings.encodeTags(lib.bioWell.bioPlate.name + ' ' + lib.bioWell.location) : '-');
		createpool.addColumn(tr, 'remain', lib.remainingVolume ? Numbers.formatNumber(lib.remainingVolume, 1) : '-');
		createpool.addColumn(tr, 'adapter', lib.libraryFracAdpt != null ? lib.libraryFracAdpt + '%' : '-');
		createpool.addColumn(tr, 'molarity', lib.molarity ? Numbers.formatNumber(lib.molarity, 2) : '-');
		createpool.addColumn(tr, 'volume', '');
		createpool.addColumn(tr, 'eb', '');
		createpool.addColumn(tr, 'status', '');
		createpool.addColumn(tr, 'remarks', '');
		tr.className = className;

		// Rows should be sorted by barcode
		// Locate existing row 
		tr.sortBy = lib.barcode ? lib.barcode.name : lib.name;
		var tbody = Doc.element('lib-table-rows');
		
		var target = null;
		for (var i = 0; i < tbody.childNodes.length; i++)
		{
			var row = tbody.childNodes[i];
			if (row.sortBy && (row.sortBy > tr.sortBy))
			{
				// Found it!
				target = row;
				break;
			};
		}
		
		tbody.insertBefore(tr, target);
	}

	createpool.addColumn = function(tr, idSuffix, text)
	{
		var td = document.createElement('td');
		td.id = tr.id+'.'+idSuffix;
		td.className = idSuffix;
		td.innerHTML = text;
		tr.appendChild(td);
	}

	
	createpool.updatePoolData = function()
	{
		if (!targetVolumePerLibIsValid || selectedLibraries.length == 0) return;
		
		var frm = document.forms['reggie'];
		var targetVolumePerLib = parseFloat(frm.target_volume.value);
		var mixingStrategy = Forms.getCheckedRadio(frm.mixing_strategy).value;
		var targetMolarityInPool = parseFloat(frm.target_molarity.value);

		var numSeparateMix = 0;
		var numLowQuantity = 0;
		var numExcluded = 0;

		// Get all libs into an array
		var libs = [];
		var usedBarcodes = [];
		var duplicateBarcodes = [];
		for (var selectedNo = 0; selectedNo < selectedLibraries.length; selectedNo++)
		{
			var lib = selectedLibraries[selectedNo];
			if (!lib.excludeFromPool)
			{
				libs[libs.length] = lib;
				if (usedBarcodes[lib.barcode.name])
				{
					duplicateBarcodes[lib.barcode.name] = 1;
				}
				else
				{
					usedBarcodes[lib.barcode.name] = 1;
				}
				PoolMix.calculateLibVolume(lib, targetMolarityInPool, targetVolumePerLib, mixingStrategy);
				PoolMix.calculateEbVolume(lib, targetMolarityInPool, targetVolumePerLib, mixingStrategy);
			}
			else
			{
				numExcluded++;
			}
		}
			
		// Calculate final pool volumes, molarity, etc.
		poolInfo = PoolMix.calculateFinalPoolInfo(libs, targetMolarityInPool, targetVolumePerLib, mixingStrategy);
		
		// Make remarks for each lib
		for (var libNo = 0; libNo < selectedLibraries.length; libNo++)
		{
			var lib = selectedLibraries[libNo];
			if (!lib.excludeFromPool)
			{
				var mark = '';
				var tr = Doc.element('lib.'+lib.id);
				if (duplicateBarcodes[lib.barcode.name] || lib.pipeline != 'RNAseq')
				{
					Doc.addClass(tr, 'lib-error');
					lib.hasError = true;
				}
				else
				{
					Doc.removeClass(tr, 'lib-error');
					lib.hasError = false;
				}
				Doc.removeClass(tr, 'warning');
				if (lib.volume != lib.actualVolume)
				{
					mark = '*';
					Doc.addClass(tr, 'warning');
				}
				if (lib.libraryFracAdpt == null || lib.libraryFracAdpt > LIBRARY_FRAC_ADPT_LIMIT)
				{
					Doc.addClass(tr, 'warning');
				}
				
				Doc.element('lib.'+lib.id+'.volume').innerHTML = lib.mixFactor > 1 ? '-' : Numbers.formatNumber(lib.volume, 1)+mark;
				if (mixingStrategy == 'fixed' && lib.mixFactor < 1.001)
				{
					Doc.element('lib.'+lib.id+'.eb').innerHTML = Numbers.formatNumber(lib.eb, 1);
				}
				else
				{
					Doc.element('lib.'+lib.id+'.eb').innerHTML = '-';
				}
				if (lib.eb != 0 || mixingStrategy == 'fixed') numSeparateMix++;
				if (lib.volume > lib.actualVolume) numLowQuantity++;
			}
			
			createpool.calculateRemarks(lib, mixingStrategy, duplicateBarcodes);
			Doc.element('lib.'+lib.id+'.remarks').innerHTML = lib.remarks.join('; ');
		}
		
		var warnMsg = null;
		if (poolInfo.ebVolumeExtra < 0) 
		{
			warnMsg = 'Too many libs with low molarity';
		}
		
		Doc.element('pool.eb').innerHTML = Numbers.formatNumber(poolInfo.ebVolumeExtra, 1);

		var poolDiv = Doc.element('pool-summary');
		var poolRow = Doc.element('lib-table-pool-summary');
		var poolData = '<div class="pool-data">';
		poolData += libs.length + ' libs • ';
		poolData += Numbers.formatNumber(poolInfo.molarity, 2, 'nM')+' • ';
		poolData += Numbers.formatNumber(poolInfo.totalVolume, 1, 'µl');
		if (mixingStrategy == 'dynamic')
		{
			poolData += ' • <span class="eb">'+Numbers.formatNumber(Math.max(0, poolInfo.ebVolumeExtra), 1, 'µl')+'</span>';
		}
		if (warnMsg)
		{
			Doc.addClass(poolRow, 'warning');
			Doc.element('pool.remarks').innerHTML = warnMsg;
		}
		else
		{
			Doc.removeClass(poolRow, 'warning');
			Doc.element('pool.remarks').innerHTML = '';
		}
		
		poolData += '</div>';
		poolDiv.innerHTML = poolData;
		
		Doc.element('num_separate_mix').innerHTML = numSeparateMix;
		Doc.element('num_low_quantity').innerHTML = numLowQuantity;
		Doc.element('num_excluded').innerHTML = numExcluded;
	}

	createpool.calculateRemarks = function(lib, mixingStrategy, duplicateBarcodes)
	{
		var remarks = [];

		if (lib.DO_NOT_USE)
		{
			remarks[remarks.length] = 'DoNotUse-'+Strings.encodeTags(lib.DO_NOT_USE + ': ' + lib.DO_NOT_USE_COMMENT);
		}
		
		if (lib.pipeline != 'RNAseq')
		{
			remarks[remarks.length] = 'Invalid pipeline: ' + Strings.encodeTags(lib.pipeline);
		}
		
		// Check if default barcode has been modified
		if (lib.molarity != null)
		{
			if (duplicateBarcodes[lib.barcode.name])
			{
				remarks[remarks.length] = 'Duplicate barcode: ' + Strings.encodeTags(lib.barcode.name);
			}
			
			if (lib.volume != lib.actualVolume)
			{
				if (lib.volume > lib.remainingVolume)
				{
					remarks[remarks.length] = 'Low quantity';
				}
				else 
				{
					remarks[remarks.length] = 'Low molarity';
				}
				if (mixingStrategy == 'fixed')
				{
					remarks[remarks.length] = 'Mix <span class="volume">'+Numbers.formatNumber(lib.actualVolume, 1) + '</span>+<span class="eb">' + Numbers.formatNumber(lib.actualEb, 1)+'µl</span>';
				}
				else
				{
					remarks[remarks.length] = 'Use <span class="volume">'+Numbers.formatNumber(lib.actualVolume, 1) + 'µl</span>';
				}
			}
			
			if (lib.libraryFracAdpt == null)
			{
				remarks[remarks.length] = 'Unknown adapter fraction';
			}
			else if (lib.libraryFracAdpt > LIBRARY_FRAC_ADPT_LIMIT)
			{
				remarks[remarks.length] = 'High adapter fraction';
			}
			
			if (lib.speedVacConc != null)
			{
				remarks[remarks.length] = 'SpeedVac';
			}
			
			if (lib.mixFactor > 1)
			{
				// Larger mix than default
				remarks[remarks.length] = 'Mix <span class="volume">'+Numbers.formatNumber(lib.volume*lib.mixFactor, 1) + '</span>+<span class="eb">' + Numbers.formatNumber(lib.eb*lib.mixFactor, 1)+'µl</span>';
				remarks[remarks.length] = 'Use <b>' + Numbers.formatNumber(lib.volume+lib.eb, 1) + 'µl</b> in pool';
			}
		}
		if (lib.excludeFromPool)
		{
			remarks[remarks.length] = 'Excluded';
		}
		if (lib.comment)
		{
			remarks[remarks.length] = Strings.encodeTags(lib.comment);
		}
		lib.stratagene = Reggie.isStratagene(lib.name);
		lib.external = Reggie.isExternal(lib.name);
		lib.remarks = remarks;
	}

	
	createpool.targetVolumeOnChange = function()
	{
		var frm = document.forms['reggie'];
		var targetVolumePerLib = parseFloat(frm.target_volume.value);
		
		targetVolumePerLibIsValid = false;
		if (targetVolumePerLib < MIN_TARGET_VOLUME || targetVolumePerLib > MAX_TARGET_VOLUME)
		{
			Wizard.setInputStatus('target_volume', 'invalid', 'Must be between '+MIN_TARGET_VOLUME+' and '+MAX_TARGET_VOLUME+'µl');
			return;
		}
		
		Wizard.setInputStatus('target_volume', 'valid');
		targetVolumePerLibIsValid = true;
		createpool.updatePoolData();
	}
	
	//Set target volume on the selected wells
	createpool.setTargetVolume = function(event)
	{
		var frm = document.forms['reggie'];
		var checked = [];
		var targetVolume;
		for (var libNo = 0; libNo < selectedLibraries.length; libNo++)
		{
			var lib = selectedLibraries[libNo];
			if (frm['check.'+lib.id].checked && lib.mixFactor > 1)
			{
				checked[checked.length] = lib;
				if (!targetVolume) targetVolume = lib.targetVolume;
			}
		}
		
		if (checked.length == 0)
		{
			Forms.showNotification(event.currentTarget, 'No libraries with separate mix have been selected');
			return;
		}

		targetVolume = parseFloat(prompt('Volume to use in pool ('+LIMIT_FOR_EXTRA_LARGE_MIX+'--'+MAX_TARGET_VOLUME+' µl)', targetVolume || lastTargetVolume || frm.target_volume.value));
		if (isNaN(targetVolume))
		{
			targetVolume = null;
		}
		else if (targetVolume > MAX_TARGET_VOLUME)
		{
			targetVolume = MAX_TARGET_VOLUME;
		}
		else if (targetVolume < LIMIT_FOR_EXTRA_LARGE_MIX)
		{
			targetVolume = LIMIT_FOR_EXTRA_LARGE_MIX
		}
		lastTargetVolume = targetVolume;
		if (targetVolume == '') targetVolume = null;

		var targetVolumePerLib = parseFloat(frm.target_volume.value);
		var mixingStrategy = Forms.getCheckedRadio(frm.mixing_strategy).value;
		var targetMolarityInPool = parseFloat(frm.target_molarity.value);
		
		for (var libNo = 0; libNo < checked.length; libNo++)
		{
			var lib = checked[libNo];
			lib.targetVolume = targetVolume;
			PoolMix.calculateEbVolume(lib, targetMolarityInPool, targetVolumePerLib, mixingStrategy);
		}
		
		createpool.updatePoolData();
	}

	//Set a comment on the checked libraries
	createpool.commentChecked = function(event)
	{
		var frm = document.forms['reggie'];
		
		// Get array with checked libs
		var checked = [];
		var comment;
		for (var libNo = 0; libNo < selectedLibraries.length; libNo++)
		{
			var lib = selectedLibraries[libNo];
			if (frm['check.'+lib.id].checked)
			{
				checked[checked.length] = lib;
				if (!comment) comment = lib.comment;
			}
		}
		
		if (checked.length == 0)
		{
			Forms.showNotification(event.currentTarget, 'No libraries have been selected');
			return;
		}
					
		comment = prompt('Comment', comment || lastComment);
		if (comment == null) return;

		lastComment = comment;
		
		if (comment == '') comment = null;
		for (var libNo = 0; libNo < checked.length; libNo++)
		{
			var lib = checked[libNo];
			lib.comment = comment;
		}
		
		createpool.updatePoolData();
	}
	
	createpool.validateStep1 = function(event)
	{
		var valid = true;
		
		valid &= targetVolumePerLibIsValid;
		valid &= (selectedLibraries.length > 0);

		var numIncluded = 0;
		for (var libNo = 0; libNo < selectedLibraries.length; libNo++)
		{
			var lib = selectedLibraries[libNo];
			valid &= !lib.hasError;
			if (!lib.excludeFromPool) numIncluded++;
		}

		valid &= (numIncluded > 0);
		
		if (!valid) event.preventDefault();
	}

	createpool.submit = function()
	{
		var frm = document.forms['reggie'];

		var submitInfo = {};
		submitInfo.pools = [];
		submitInfo.flagged = []; // Always empty
		var mixingStrategy = Forms.getCheckedRadio(frm.mixing_strategy).value;
		
		var pool = {};
		pool.name = frm.poolName.value;
		pool.comment = frm.poolComments.value;
		pool.ebVolumeExtra = Math.max(0, poolInfo.ebVolumeExtra);
		pool.targetVolumeInPoolPerLib = parseFloat(frm.target_volume.value);
		pool.targetPoolMolarity = parseFloat(frm.target_molarity.value);
		pool.mixingStrategy = mixingStrategy;
		pool.libs = [];
		pool.excluded = [];
		
		for (var libNo = 0; libNo < selectedLibraries.length; libNo++)
		{
			var lib = selectedLibraries[libNo];
			
			// We send library id, name and mixing volumes
			if (!lib.excludeFromPool)
			{
				var tmp = {};
				tmp.id = lib.id;
				tmp.name = lib.name;
				tmp.volume = lib.actualVolume;
				tmp.eb = lib.actualEb;
				tmp.mixFactor = lib.mixFactor;
				if (lib.mixFactor > 1 && lib.targetVolume && mixingStrategy == 'dynamic') 
				{
					tmp.targetVolume = lib.targetVolume;
				}
				tmp.comment = lib.comment;
				pool.libs[pool.libs.length] = tmp;
			}
		}
		submitInfo.pools[submitInfo.pools.length] = pool;
		
		var url = '../Pool.servlet?ID='+App.getSessionId();
		url += '&cmd=CreatePools';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, createpool.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	createpool.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return createpool;
}();

Doc.onLoad(CreatePool.initPage);

