var Methylation = function()
{
	var meth = {};
	var debug = 0;
	var selectionIsValid = false;
	var subtypeMethylation = null;
	var annotationTypePipeline = null;
	
	var clusterIsValid = false;
	var manuallySelected = [];
	
	// Page initialization
	meth.initPage = function()
	{
		// Step 1
		Buttons.addClickHandler('btnSelectMethylation', meth.selectMethylationItems);
		Events.addEventHandler('methylationItems', 'base-selected', meth.setMethylationItemsCallback);
		Events.addEventHandler('methylationItems', 'change', meth.methylationItemsOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', meth.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', meth.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', meth.validateStep2);
		Events.addEventHandler('clusters', 'change', meth.clusterOnChange);
		Events.addEventHandler('betaSoftware', 'change', meth.softwareOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', meth.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Methylation.servlet?ID='+App.getSessionId();
		url += '&cmd=GetMethylationItemsForBetaAnalysis';
		Wizard.asyncJsonRequest(url, meth.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	meth.initializeStep1 = function(response)
	{
		var methylationItems = response.methylationItems;
		var frm = document.forms['reggie'];	

		if (methylationItems != null && methylationItems.length > 0)
		{
			var firstChip = null;
			for (var mNo=0; mNo < methylationItems.length; mNo++)
			{
				var m = methylationItems[mNo];
				var beadChip = m.beadChip ? m.beadChip : null; 
				if (firstChip == null && beadChip != null) firstChip = beadChip.name;
				var option = meth.createListOption(mNo+1, m, m.DO_NOT_USE == null && beadChip != null && beadChip.name==firstChip)
				frm.methylationItems.options[frm.methylationItems.length] = option;
			}
			Events.sendChangeEvent('methylationItems');
		}
		else
		{
			Wizard.setInputStatus('methylationItems', 'invalid', 'No new methylation items available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.methylationItems.focus();
	}
	
	meth.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}
	
	meth.selectMethylationItems = function()
	{
		var frm = document.forms['reggie'];
		if (frm.methylationItems.disabled) return;
		
		if (subtypeMethylation == null)
		{
			subtypeMethylation = Reggie.getSubtypeInfo('METHYLATION');
		}
		if (annotationTypePipeline == null)
		{
			annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		}
		
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeMethylation.id;
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'='+encodeURIComponent('DNA/Methylation');

		Dialogs.selectItem('DERIVEDBIOASSAY', 'methylationItems', 1, url);
	}

	meth.setMethylationItemsCallback = function(event)
	{
		var m = event.detail;
		var opt = Reggie.getListOption('methylationItems', m.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = m.id;
		}
		
		if (m.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../Methylation.servlet?ID='+App.getSessionId();
			url += '&cmd=GetMethylationItemsForBetaAnalysis';
			url += '&items='+manuallySelected.join(',');
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, meth.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('methylationItems');
		}
	}

	meth.manuallySelected = function(response)
	{
		var methylationItems = response.methylationItems;
		var frm = document.forms['reggie'];	

		if (methylationItems != null && methylationItems.length > 0)
		{
			var offset = frm.methylationItems.length+1;
			for (var mNo=0; mNo < methylationItems.length; mNo++)
			{
				var m = methylationItems[mNo];
				var option = meth.createListOption(mNo+offset, m, m.DO_NOT_USE == null);
				frm.methylationItems.options[frm.methylationItems.length] = option;
			}
			Events.sendChangeEvent('methylationItems');
		}
	}

	meth.createListOption = function(index, methylationItem, selected)
	{
		var dna = methylationItem.dna;
		var chip = methylationItem.beadChip;
		
		var tooltip = null;
		var name = (index) + ': '+methylationItem.name;
		name += ' - '+chip.BeadChipId+'_'+dna.BeadChipPosition;
		
		if (methylationItem.DO_NOT_USE)
		{
			name += ' [DoNotUse]';
			tooltip = 'DoNotUse-'+Strings.encodeTags(methylationItem.DO_NOT_USE+': '+methylationItem.DO_NOT_USE_COMMENT);
		}
		else if (methylationItem.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		var option = new Option(name, methylationItem.id, false, selected);
		if (tooltip) option.title = tooltip;
		option.methylationItem = methylationItem;
		return option;
	}
	
	meth.methylationItemsOnChange = function()
	{
		var frm = document.forms['reggie'];
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;
		Wizard.setInputStatus('methylationItems');
		Wizard.hideGoNextConfirmation();
		
		for (var mNo = 0; mNo < frm.methylationItems.length; mNo++)
		{
			if (frm.methylationItems[mNo].selected) 
			{
				numSelected++;
				var m = frm.methylationItems[mNo].methylationItem;
				if (m.DO_NOT_USE) numDoNotUse++;
				if (m.pipeline && m.pipeline != 'DNA/Methylation')
				{
					invalidPipeline = Strings.encodeTags(m.name+' ('+m.pipeline)+') is not intended for the DNA/Methylation pipeline.';
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('methylationItems', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('methylationItems', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('methylationItems', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('methylationItems', 'valid');
		}
	}

	// --- Step 2 -----------------------------------
	meth.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadSoftware('METHYLATION_SOFTWARE', 'betaSoftware', 'PARAMETER_SET');
		
		// Load clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config=methylation-beta';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, meth.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	meth.validateStep2 = function(event)
	{
		if (!clusterIsValid) event.preventDefault();
	}
	
	meth.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available clusters');
			clusterIsValid = false;
		}
		else
		{
			meth.clusterOnChange();
		}
	}
	
	meth.softwareOnChange = function(event)
	{
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, meth.parameterSetInfoLoaded);
		}
	}
	
	meth.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	meth.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	meth.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.betaSoftware = parseInt(frm.betaSoftware.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
			
		var methylationItems = [];
		submitInfo.methylationItems = methylationItems;
			
		for (var mNo = 0; mNo < frm.methylationItems.length; mNo++)
		{
			if (frm.methylationItems[mNo].selected) 
			{
				var m = {};
				m.id = frm.methylationItems[mNo].methylationItem.id;
				methylationItems[methylationItems.length] = m;
			}
		}

		var url = '../Methylation.servlet?ID='+App.getSessionId();
		url += '&cmd=StartBetaAnalysis';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, meth.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	meth.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return meth;
}();

Doc.onLoad(Methylation.initPage);

