var Blood = function()
{
	var blood = {};
	var debug = 0;
	
	var patientInfo = null;
	var bloodInfo = null;
	var consentInfo = null;
	var caseInfo = null;

	var rccidIsValid = false;
	var pnrIsValid = false;
	var pnrIsAcceptable = false;

	var samplingDateIsValid = true;
	var freezerDateIsValid = true;

	blood.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', blood.validateStep1);
		Events.addEventHandler('rccidNumber', 'keypress', Wizard.focusOnEnter);
		Events.addEventHandler('rccidNumber', 'blur', blood.rccidNumberOnChange);
		Events.addEventHandler('personalNumber', 'keyup', blood.personalNumberOnChange);
		Events.addEventHandler('personalNumber', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', blood.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', blood.validateStep2);
		
		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', blood.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', blood.validateStep3);
		Events.addEventHandler('samplingDate', 'blur', blood.samplingDateTimeOnChange);
		Events.addEventHandler('samplingTime', 'blur', blood.samplingDateTimeOnChange);
		Events.addEventHandler('freezerDate', 'blur', blood.freezerDateTimeOnChange);
		Events.addEventHandler('freezerTime', 'blur', blood.freezerDateTimeOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('goupdate', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', blood.submit);
		
		Doc.show('step-1');
		Doc.show('gonext');
	}
	

	blood.rccidNumberOnChange = function()
	{
		var frm = document.forms['reggie'];
		Wizard.setInputStatus('rccidNumber');
		
		var rccidNumber = frm.rccidNumber.value.toUpperCase();
		rccidIsValid = false;
		if (rccidNumber == '')
		{
			Wizard.setInputStatus('rccidNumber', 'invalid', 'Missing');
			frm.rccidNumber.focus();
			return;
		}
		
		if (!Reggie.isValidRccidNumber(rccidNumber))
		{
			Wizard.setInputStatus('rccidNumber', 'invalid', 'Expected NNNN-NNNNNNNN where N is a digit, or 10 digits + \'B\', \'C\', or \'D\'');
			frm.rccidNumber.focus();
			return;
		}

		frm.rccidNumber.value = rccidNumber;
		Wizard.setInputStatus('rccidNumber', 'valid');
		rccidIsValid = true;


		var url = '../BloodFollowUpRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetBloodFollowUpInfo';
		url += '&rccidNumber=' + encodeURIComponent(rccidNumber);
		Wizard.showLoadingAnimation('Loading information follow-up blood...');
		Wizard.asyncJsonRequest(url, blood.bloodInfoLoadedFromRccid);
	}
	
	blood.bloodInfoLoadedFromRccid = function(response)
	{
		var frm = document.forms['reggie'];
		bloodInfo = response.bloodInfo;
		patientInfo = response.patientInfo;
		consentInfo = response.consentInfo;
		caseInfo = response.caseInfo;
		
		if (patientInfo)
		{
			// Move to next step
			frm.personalNumber.value = patientInfo.personalNumber;
			blood.personalNumberOnChange();
			Wizard.goNext(false);
		}
	}
	
	blood.personalNumberOnChange = function()
	{
		var frm = document.forms['reggie'];
		var pnr = frm.personalNumber.value;
		pnrIsValid = false;
		pnrIsAcceptable = true;
		
		var pnrLen = pnr.length;
		if (pnrLen != 12 && pnrLen != 10)
		{
			Wizard.setInputStatus('personalNumber', 'warning', pnrLen < 12 ? 'Too short' : 'Too long');
			return;
		}

		if (!pnr.match(/^\d+$/))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Non-standard personal number');
			return;
		}
		
		var datePart = pnr.substring(0, pnrLen - 4); // Remove last four digits
		var dateFormat = pnrLen == 10 ? 'yyMMdd' : 'yyyyMMdd';
		var isValidDate = Dates.isDate(datePart, dateFormat);
		if (!isValidDate)
		{
			var dayInMonth = parseInt(datePart.substring(-2), 10); // Last two characters
			if (dayInMonth > 60)
			{
				// 'Samordningsnummer' has day-in-month + 60; check this
				dayInMonth -= 60;
				var tmpPnr = pnr.substring(0, pnrLen - 6); // First 4 or 6 digits is year+month
				if (dayInMonth < 10) tmpPnr += '0';
				tmpPnr += dayInMonth;
				isValidDate = Dates.isDate(tmpPnr, dateFormat);
			}
		}
		if (!isValidDate)
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Not a valid date');
			return;
		}
		
		if (!Reggie.personalNumberControlDigitCheck(pnr.substr(pnrLen == 10 ? 0 : 2)))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Invalid control digit');
			return;
		}
		
		Wizard.setInputStatus('personalNumber', 'valid');
		pnrIsValid = true;
		Wizard.hideGoNextConfirmation();
	}

	blood.validateStep1 = function(event)
	{
		App.debug('gonext:'+event.detail.auto);
		if (event.detail.auto)
		{
			if (!rccidIsValid || !pnrIsValid) 
			{
				if (pnrIsAcceptable)
				{
					var frm = document.forms['reggie'];
					frm.personalNumber.focus();
					Wizard.showGoNextConfirmation(true, 'Check to verify registration of non-standard personal number.');
				}
				event.preventDefault();
			}
		}
		else
		{
			if (!rccidIsValid || !pnrIsAcceptable) event.preventDefault();
		}

	}
	
	blood.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		if (!patientInfo || !patientInfo.id)
		{
			var pnr = frm.personalNumber.value;
			if (pnr.length == 10 && pnrIsValid) 
			{
				var year = Reggie.fullYear(parseInt(pnr.substr(0, 2), 10));
				pnr = year + pnr.substr(2);
			}
			var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
			url += '&cmd=GetPatientInfo';
			url += '&personalNumber=' + encodeURIComponent(pnr);
			url += '&pnrIsValid=' + pnrIsValid;
			
			Wizard.showLoadingAnimation('Loading patient information...');
			Wizard.asyncJsonRequest(url, blood.patientInfoLoaded);
		}
		else
		{
			blood.patientInfoLoaded();
		}
	}
	
	blood.patientInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		if (response) patientInfo = response.patientInfo;
		if (!patientInfo || !patientInfo.id)
		{
			Wizard.setFatalError('No patient found for personal number: ' + Strings.encodeTags(frm.personalNumber.value));
			return;
		}
	
		Doc.element('existing.patientCode').innerHTML = Strings.encodeTags(patientInfo.name);
		Doc.element('existing.familyName').innerHTML = Strings.encodeTags(patientInfo.familyName);
		Doc.element('existing.allFirstNames').innerHTML = Strings.encodeTags(patientInfo.allFirstNames);
		if (patientInfo.gender) 
		{
			Doc.element('existing.gender').innerHTML = Strings.encodeTags(patientInfo.gender);
		}
		if (patientInfo.dateOfBirth) 
		{
			Doc.element('existing.dateOfBirth').innerHTML = Strings.encodeTags(patientInfo.dateOfBirth);
		}
		
		// Get case option list JSON object from personal number with AJAX
		var url = '../BloodFollowUpRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseOptions';
		url += '&patientId='+patientInfo.id;
		if (caseInfo && caseInfo.id)
		{
			url += '&caseId='+caseInfo.id;
		}
		
		Wizard.showLoadingAnimation('Loading case information...');
		Wizard.asyncJsonRequest(url, blood.caseInfoLoaded);
	}
	
	blood.caseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var caseOptions = response.caseOptionArray || [];
		// Construct case name menu from JSON data
		for (var i = 0; i < caseOptions.length; i++)
		{
			var option = caseOptions[i];
			frm.caseName[frm.caseName.length] = new Option(option.text, option.id);
		}
		Wizard.setCurrentStep(2);

		if (caseOptions.length == 0)
		{
			Wizard.setFatalError('Can\'t find any case for this patient');
			return;
		}
		else if (caseOptions.length == 1)
		{
			// Only one case, go directly to blood data entries
			Wizard.setInputStatus('caseName', 'valid', 'Only one case found');
			Wizard.goNext(true);
			return;
		}
		else if (caseOptions.length > 1)
		{
			// More than one case, let user select case to use
			Wizard.setInputStatus('caseName', 'valid', caseOptions.length + ' cases found. Select case to use.')
		}
		
		Doc.show('gonext');
		Doc.show('gocancel');
	}
	
	// Nothing to validate
	blood.validateStep2 = function(event)
	{}
	
	
	blood.initializeStep3 = function()
	{
		var frm = document.forms['reggie'];

		var caseId = frm.caseName.value;
		
		if (!bloodInfo)
		{
			var url = '../BloodFollowUpRegistration.servlet?ID='+App.getSessionId();
			url += '&cmd=GetNextBloodName';
			url += '&caseId=' + caseId;

			Wizard.showLoadingAnimation('Loading blood information...');
			Wizard.asyncJsonRequest(url, blood.bloodInfoLoadedFromCase);
		}
		else
		{
			blood.bloodInfoLoadedFromCase();
		}
	}
	
	blood.bloodInfoLoadedFromCase = function(response)
	{
		var frm = document.forms['reggie'];
		if (response) 
		{
			bloodInfo = response.bloodInfo;
			consentInfo = response.consentInfo;
		}
		
		var updateMode = bloodInfo.id;
		
		if (updateMode)
		{
			Doc.element('step-3-title').innerHTML = 'Update blood \''+Strings.encodeTags(bloodInfo.name)+'\'';
			if (bloodInfo.samplingDate != null)
			{
				frm.samplingDate.value = bloodInfo.samplingDate.substring(0, 8);
				frm.samplingTime.value = bloodInfo.samplingDate.substring(9, 13);
				blood.samplingDateTimeOnChange();
			}
			if (bloodInfo.freezerDate != null)
			{
				frm.freezerDate.value = bloodInfo.freezerDate.substring(0, 8);
				frm.freezerTime.value = bloodInfo.freezerDate.substring(9, 13);
				blood.freezerDateTimeOnChange();
			}
			frm.serum.checked = bloodInfo.serum == 'Yes';
			if (bloodInfo.bloodSample)
			{
				Forms.selectListOption(frm.bloodSample, bloodInfo.bloodSample);
			}
			if (bloodInfo.otherPathNote)
			{
				frm.otherPathNote.value = bloodInfo.otherPathNote;
			}
		}
		else
		{
			frm.serum.checked = true;
			frm.serum.disabled = true;
			// Select follow-up blood annotation from RCCID suffix letter (last character in name)
			var rccidNumber = frm.rccidNumber.value;
			var rccidSuffixLetter = rccidNumber.substring(rccidNumber.length-1, rccidNumber.length);
			var followUpCode = null;
			if (rccidSuffixLetter == 'B')
			{
				followUpCode = 'FollowUp06';
			}
			else if (rccidSuffixLetter == 'C')
			{
				followUpCode = 'FollowUp12';
			}
			else if (rccidSuffixLetter == 'D')
			{
				followUpCode = 'FollowUp36';
			}
			if (followUpCode != null)
			{
				Forms.selectListOption(frm.bloodSample, followUpCode);
				frm.bloodSample.disabled = true;
			}
			if (consentInfo && consentInfo.consent)
			{
				frm.copyConsent.value = consentInfo.id;
				var html = "From '" + Strings.encodeTags(consentInfo.name) + "' (" + consentInfo.consent;
				if (consentInfo.consentDate) html += '; ' + consentInfo.consentDate;
				html += ')';
				Doc.element('copyConsent.text').innerHTML =  html;
				Doc.show("copyConsentSection");
			}
		}
		
		if (updateMode)
		{
			Doc.show('goupdate');
		}
		else
		{
			Doc.show('goregister');
		}
		Doc.show('gocancel');
		Wizard.setCurrentStep(3);
		frm.samplingDate.focus();
	}
	
	blood.validateStep3 = function(event)
	{
		if (!samplingDateIsValid || !freezerDateIsValid)
		{
			event.preventDefault();
		}
	}

	blood.samplingDateTimeOnChange = function()
	{
		var frm = document.forms['reggie'];
		samplingDateIsValid = false;
		Wizard.setInputStatus('samplingDate');
		
		var samplingDate = frm.samplingDate.value;
		var samplingTime = frm.samplingTime.value;
		
		if (samplingDate != '' || samplingTime != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			samplingDate = Reggie.autoFillDate(samplingDate, true);
			frm.samplingDate.value = samplingDate;

			samplingTime = Reggie.autoFillTime(samplingTime);
			frm.samplingTime.value = samplingTime;
			
			if (!Dates.isDate(samplingDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid date');
				return;
			}
			if (frm.freezerDate.value == '') frm.freezerDate.value = samplingDate;
			
			if (!Dates.isDate(samplingDate + ' ' + samplingTime, 'yyyyMMdd HHmm'))
			{
				if (samplingDate != '') 
				{
					Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid time');
				}
				return;
			}
			Wizard.setInputStatus('samplingDate', 'valid');
		}
		samplingDateIsValid = true;
	}

	blood.freezerDateTimeOnChange = function()
	{
		var frm = document.forms['reggie'];
		freezerDateIsValid = false;
		Wizard.setInputStatus('freezerDate');
		
		var freezerDate = frm.freezerDate.value;
		var freezerTime = frm.freezerTime.value;
		var freezerTimestamp;
		
		if (freezerDate != '' || freezerTime != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			freezerDate = Reggie.autoFillDate(freezerDate, true);
			frm.freezerDate.value = freezerDate;

			freezerTime = Reggie.autoFillTime(freezerTime);
			frm.freezerTime.value = freezerTime;
			
			if (!Dates.isDate(freezerDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('freezerDate', 'invalid', 'Not a valid date');
				return;
			}
			freezerTimestamp = Dates.parseString(freezerDate + ' ' + freezerTime, 'yyyyMMdd HHmm');
			if (freezerTimestamp == null)
			{
				if (freezerDate != '') 
				{
					Wizard.setInputStatus('freezerDate', 'invalid', 'Not a valid time');
				}
				return;
			}
			Wizard.setInputStatus('freezerDate', 'valid');
		}
		
		freezerDateIsValid = true;

		if (samplingDateIsValid && freezerTimestamp)
		{
			var samplingTimestamp = Dates.parseString(frm.samplingDate.value + ' ' + frm.samplingTime.value, 'yyyyMMdd Hmm');
			if (samplingTimestamp && samplingTimestamp.getDate() != freezerTimestamp.getDate())
			{
				Wizard.setInputStatus('freezerDate', 'warning', 'Sampling and freezer dates are different');
			}
		}
	}


	blood.submit = function()
	{
		var frm = document.forms['reggie'];

		bloodInfo.rccidNumber = frm.rccidNumber.value;
		bloodInfo.samplingDate = Strings.trim(frm.samplingDate.value + ' ' + frm.samplingTime.value);
		bloodInfo.freezerDate = Strings.trim(frm.freezerDate.value + ' ' + frm.freezerTime.value);
		bloodInfo.serum = "Yes";
		bloodInfo.bloodSample = frm.bloodSample.value;
		bloodInfo.otherPathNote = frm.otherPathNote.value;
		if (frm.copyConsent.checked && frm.copyConsent.value)
		{
			bloodInfo.copyConsent = parseInt(frm.copyConsent.value, 10);
		}

		var updateMode = bloodInfo.id;
		var submitInfo = {};
		submitInfo.patientInfo = patientInfo;
		submitInfo.bloodInfo = bloodInfo;

		var url = '../BloodFollowUpRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=' + (updateMode ? 'UpdateBlood' : 'CreateBlood');

		Wizard.showLoadingAnimation('Registering blood information...');
		Wizard.asyncJsonRequest(url, blood.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	blood.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	return blood;
}();

Doc.onLoad(Blood.initPage);

