var Consent = function()
{
	var consent = {};
	var consentToId = { "Yes": "consent.yes", "No" : "consent.no", "Not asked": "consent.notAsked" };
	var patientInfo;
	var debug = 0;
	
	consent.initPage = function()
	{
		
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', consent.validateStep1);
		Events.addEventHandler('caseName', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', consent.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', consent.validateStep2);
		Events.addEventHandler('consentDate', 'blur', Wizard.validateDate);
		
		Events.addEventHandler('consent.yes', 'change', consent.consentOnChange);
		Events.addEventHandler('consent.no', 'change', consent.consentOnChange);
		Events.addEventHandler('consent.notAsked', 'change', consent.consentOnChange);
		Events.addEventHandler('otherReasonIfNotAsked', 'focus', consent.onOtherReasonFocus);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', consent.submit);
		
		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	/**
		Check that the case number is valid.
	*/
	consent.validateStep1 = function(event)
	{
		var frm = document.forms['reggie'];
		
		var caseName = frm.caseName.value;
		var caseIsValid = false;
		Wizard.setInputStatus('caseName');
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
		}
		else if (!Reggie.isValidCaseName(caseName, false)) // 'C' suffix is not used
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Only 7-digits name is allowed .');
			frm.caseName.focus();
		}
		else
		{
			Wizard.setInputStatus('caseName', 'valid');
			caseIsValid = true;
		}
		
		// If not valid, we prevent the wizard from moving to the next step
		if (!caseIsValid) event.preventDefault();
	}
	
	// Check that the date is valid
	consent.validateStep2 = function(event)
	{
		var consentDateIsValid = Wizard.isValid('consentDate');
		if (!consentDateIsValid) event.preventDefault();
	}
	
	/**
		Load information about the given case.
	*/
	consent.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;

		var url = '../ConsentForm.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo&caseName=' + encodeURIComponent(caseName);
		
		Wizard.showLoadingAnimation('Loading case information...');
		Wizard.asyncJsonRequest(url, consent.caseInfoLoaded);
	}
	
	/**
		Initalize the second step based on the information we have about the case.
	*/
	consent.caseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;
		patientInfo = response.patientInfo;
		var caseInfo = response.caseInfo;
		var specimenInfo = response.specimenInfo;

		if (patientInfo)
		{
			var moreCasesHtml = '';
			var numMissing = 0;
			var notYesConsent = null;
			if (patientInfo.allCases)
			{
				for (var i = 0; i < patientInfo.allCases.length; i++)
				{
					var c = patientInfo.allCases[i];
					var moreInfo = [];
					if (c.laterality) moreInfo[moreInfo.length] = c.laterality;
					if (c.consentDate) moreInfo[moreInfo.length] = Reggie.reformatDate(c.consentDate);
					var info = moreInfo.length == 0 ? '' : ' (' + moreInfo.join(', ') + ')';
					var checked = '';
					if (!c.consent)
					{
						checked = 'checked';
						numMissing++;
					}
					else if (c.consent != 'Yes')
					{
						notYesConsent = c.consent;
					}
					if (c.name == caseName && c.consentDate)
					{
						frm.consentDate.value = c.consentDate;
						checked = 'checked';
					}
					moreCasesHtml += '<input type="checkbox" name="case.'+c.id+'" id="case.'+c.id+'" ' + checked +'>';
					moreCasesHtml += '<label for="case.'+c.id+'">Case: ' + Strings.encodeTags(c.name + info) + '</label><br>';
				}
			}
			
			if (patientInfo.allBlood)
			{
				for (var i = 0; i < patientInfo.allBlood.length; i++)
				{
					var c = patientInfo.allBlood[i];
					var moreInfo = [];
					if (c.consentDate) moreInfo[moreInfo.length] = Reggie.reformatDate(c.consentDate);
					var info = moreInfo.length == 0 ? '' : ' (' + moreInfo.join(', ') + ')';
					var checked = '';
					if (!c.consent)
					{
						checked = 'checked';
						numMissing++;
					}
					else if (c.consent != 'Yes')
					{
						notYesConsent = c.consent;
					}
					if (c.name.indexOf(caseName) == 0 && c.consentDate)
					{
						frm.consentDate.value = c.consentDate;
						checked = 'checked';
					}
					moreCasesHtml += '<input type="checkbox" name="blood.'+ c.id + '" id="blood.'+c.id+'" ' + checked +'>';
					moreCasesHtml += '<label for="blood.'+c.id+'">Blood: ' + Strings.encodeTags(c.name + info) + '</label><br>';
				}
			}

			if (notYesConsent)
			{
				consent.enableConsentOption(consentToId[notYesConsent], true);
				Doc.element('moreCasesNo').innerHTML = moreCasesHtml;
				Doc.show('moreCasesSectionNo');
				Wizard.setInputStatus('consent', 'valid', '"' + notYesConsent + '" has already been registered for this case. It is not possible to change with this wizard.');
				Doc.show('gorestart');
				Wizard.setNoConfirm(true);
			}
			else
			{
				consent.enableConsentOption('consent.yes', true);
				Doc.element('moreCases').innerHTML = moreCasesHtml;
				Doc.show('moreCasesSection');
				if (numMissing > 0)
				{
					Wizard.setInputStatus('consent', 'valid', 'This patient already exists and has at least one case or blood sample with missing consent.');
				}
				else
				{
					Wizard.setInputStatus('consent', 'valid', '"Yes" has already been registered for all case and blood samples for this patient.');
				}
				Doc.show('morehelp.multiple', 'inline');
				Doc.show('morehelp.yesonly', 'inline');
				Doc.show('gocancel');
				Doc.show('goregister');
			}
		}
		else if (caseInfo)
		{
			// A consent has already been registered for this case
			// We do not support updating this
			consent.enableConsentOption(consentToId[caseInfo.consent], true);
			Wizard.setInputStatus('consent', '', '"' + caseInfo.consent + '" has already been registered for this case. It is not possible to change with this wizard.');
			Doc.show('gorestart');
			Wizard.setNoConfirm(true);
			if (caseInfo.consentDate)
			{
				frm.consentDate.value = caseInfo.consentDate;
			}
			frm.consentDate.disabled = true;
		}
		else if (specimenInfo)
		{
			// There is no existing case but there are specimen related to it
			// This wizard will only allow 'Yes' option on the consent form
			consent.enableConsentOption('consent.yes', true);
			Wizard.setInputStatus('consent', 'valid', 'Specimen tubes for this case already exists.');
			Doc.show('morehelp.yesonly', 'inline');
			Doc.show('gocancel');
			Doc.show('goregister');
		}
		else
		{
			// We have no information about this case -- but allow a consent to be registered in any case
			consent.enableConsentOption('consent.no', true);
			consent.enableConsentOption('consent.notAsked');
			consent.enableConsentOption('consent.yes');
			Doc.show('morehelp.any', 'inline');
			Doc.show('gocancel');
			Doc.show('goregister');
		}

		Wizard.setCurrentStep(2);
		if (!frm.consentDate.disabled) frm.consentDate.focus();
		
	}
	
	
	consent.submit = function()
	{
		
		var frm = document.forms['reggie'];

		var consentInfo = {};
		consentInfo.caseName = frm.caseName.value;
		consentInfo.consentDate = frm.consentDate.value;
		consentInfo.consent = Forms.getCheckedRadio(frm.consent).value;
		if (consentInfo.consent == 'Not asked')
		{
			var reasonIfNotAsked = '';
			for (var i = 0; i < frm.reasonIfNotAsked.length; i++)
			{
				var option = frm.reasonIfNotAsked[i];
				if (option.checked)
				{
					if (option.id == 'notAsked.other')
					{
						reasonIfNotAsked += frm.otherReasonIfNotAsked.value;
					}
					else
					{
						var label = Doc.element(option.id + '.label');
						reasonIfNotAsked += label.innerHTML + '\n';
					}
				}
			}
			consentInfo.reasonIfNotAsked = reasonIfNotAsked;
		}
		
		if (patientInfo)
		{
			var selectedSamples = [];
			if (patientInfo.allCases)
			{
				for (var i = 0; i < patientInfo.allCases.length; i++)
				{
					var c = patientInfo.allCases[i];
					var chk = frm['case.'+c.id];
					if (chk.checked) selectedSamples.push(c.id);
				}
			}
			
			if (patientInfo.allBlood)
			{
				for (var i = 0; i < patientInfo.allBlood.length; i++)
				{
					var c = patientInfo.allBlood[i];
					var chk = frm['blood.'+c.id];
					if (chk.checked) selectedSamples.push(c.id);
				}
			}
			consentInfo.selectedSamples = selectedSamples;
		}

		var submitInfo = {};
		submitInfo.consentInfo = consentInfo;
		
		var url = '../ConsentForm.servlet?ID=' + App.getSessionId();
		url += '&cmd=RegisterConsent';
		
		Wizard.showLoadingAnimation('Registering consent information...');
		Wizard.asyncJsonRequest(url, consent.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	
	consent.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	consent.enableConsentOption = function(option, checkIt)
	{
		option = Doc.element(option); // The radio button
		option.disabled = false;
		if (checkIt) option.checked = true;
		
		var label = Doc.element(option.id + '.label'); // The label
		Doc.removeClass(label, 'disabled');
	}



	consent.consentOnChange = function()
	{
		// If "Not asked" is selected we must enable the second question
		var isAsked = !Doc.element('consent.notAsked').checked;
		
		var frm = document.forms['reggie'];
		var reasonIfNotAsked = frm.reasonIfNotAsked;
		for (var i = 0; i < reasonIfNotAsked.length; i++)
		{
			reasonIfNotAsked[i].disabled = isAsked;
			Doc.addOrRemoveClass(reasonIfNotAsked[i].id + '.label', 'disabled', isAsked);
		}
		frm.otherReasonIfNotAsked.disabled = isAsked;		
	}
	
	consent.onOtherReasonFocus = function()
	{
		Doc.element('notAsked.other').checked = true;
	}

	return consent;
}();

Doc.onLoad(Consent.initPage);

