var RetractionForm = function()
{
	var rf = {};
	var patientInfo;
	var numItemsToBeDestroyed = 0;
	var debug = 0;
	var TRUNCATE_SIZE = [-1, 30, 20, 15];
	
	rf.initPage = function()
	{
		previewTitle = Doc.element('previewTitle');
		previewList = Doc.element('previewList');
		
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', rf.validateStep1);
		Events.addEventHandler('caseName', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', rf.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', rf.validateStep2);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);		
		Buttons.addClickHandler('gocreate', rf.createProtocol);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', rf.submit);
		
		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	/**
		Check that the case number is valid.
	*/
	rf.validateStep1 = function(event)
	{
		var frm = document.forms['reggie'];
		
		var caseName = frm.caseName.value;
		var caseIsValid = false;
		Wizard.setInputStatus('caseName');
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
		}
		else if (!Reggie.isValidCaseName(caseName, false)) // 'C' suffix is not used
		{
			rf.validatePersonalNumber(caseName);
			if (pnrIsValid)
			{
				caseIsValid = true;
			}
			else
			{
				// Entered number is neither a valid case name or personal number
				// Try to give as appropriate error message as possible
				if (caseName.length < 7 || pnrErrorMsg == null || pnrErrorMsg == '')
				{
					// The entered number is probably a faulty case name
					Wizard.setInputStatus('caseName', 'invalid', 'Only 7-digits name is allowed .');
				}
				else
				{
					Wizard.setInputStatus('caseName', 'invalid', pnrErrorMsg);
				}
				frm.caseName.focus();
			}
		}
		else
		{
			Wizard.setInputStatus('caseName', 'valid');
			caseIsValid = true;
		}
		
		// If not valid, we prevent the wizard from moving to the next step
		if (!caseIsValid) event.preventDefault();
	}
	
	rf.validatePersonalNumber = function(pnr)
	{
		//var frm = document.forms['reggie'];
		//var pnr = frm.personalNumber.value;
		pnrIsValid = false;
		pnrIsAcceptable = true;
		pnrErrorMsg = null;
		
		var pnrLen = pnr.length;
		if (pnrLen != 12 && pnrLen != 10)
		{
			Wizard.setInputStatus('caseName', 'warning', pnrLen < 12 ? 'Personal number too short' : 'Personal number too long');
			if (pnrLen < 12)
			{
				pnrErrorMsg = 'Personal number too short';
			}
			else
			{
				pnrErrorMsg = 'Personal number too long';
			}
			return;
		}

		if (!pnr.match(/^\d+$/))
		{
			if (!pnr.match(/^\d+[A-Za-z]$/))
			{
				return;
			}
		}
		
		var datePart = pnr.substring(0, pnrLen - 4); // Remove last four digits
		var dateFormat = pnrLen == 10 ? 'yyMMdd' : 'yyyyMMdd';
		var isValidDate = Dates.isDate(datePart, dateFormat);
		if (!isValidDate)
		{
			var dayInMonth = parseInt(datePart.substring(-2), 10); // Last two characters
			if (dayInMonth > 60)
			{
				// 'Samordningsnummer' has day-in-month + 60; check this
				dayInMonth -= 60;
				var tmpPnr = pnr.substring(0, pnrLen - 6); // First 4 or 6 digits is year+month
				if (dayInMonth < 10) tmpPnr += '0';
				tmpPnr += dayInMonth;
				isValidDate = Dates.isDate(tmpPnr, dateFormat);
			}
		}
		if (!isValidDate)
		{
			Wizard.setInputStatus('caseName', 'warning', 'Personal number: Not a valid date');
			pnrErrorMsg = 'Personal number: Not a valid date';
			return;
		}
		
		if (!pnr.match(/^\d+[A-Za-z]$/))
		{
			if (!Reggie.personalNumberControlDigitCheck(pnr.substr(pnrLen == 10 ? 0 : 2)))
			{
				Wizard.setInputStatus('caseName', 'warning', 'Personal number: Invalid control digit');
				pnrErrorMsg = 'Personal number: Invalid control digit';
				return;
			}
		}

		Wizard.setInputStatus('caseName', 'valid');
		pnrIsValid = true;
	}

	rf.validateStep2 = function(event)
	{}
	
	/**
		Load information about the given case.
	*/
	rf.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;

		var url = '../Retraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo&caseName=' + encodeURIComponent(caseName);
		
		Wizard.showLoadingAnimation('Loading case information...');
		Wizard.asyncJsonRequest(url, rf.caseInfoLoaded);
	}
	
	/**
		Initalize the second step based on the information we have about the case.
	*/
	rf.caseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;
		if (!response.patient && !response.caseInfo)
		{
			Wizard.setFatalError('Could not find any patient or case information for ' + caseName);
			return;
		}
		patientInfo = response.patient;
		var caseInfo = response.caseInfo;
		var caseNames = response.allCaseNames;

		var html = '<table id="retractionItemsTable">';

		// Patient Header row
		html += '<thead class="bg-filled-100">';
		html += '<tr>';
		html += '<th colspan="2">Patient</th>';
		html += '<th class="dottedleft" colspan="2">Date</th>';
		html += '<th class="dottedleft">Gender</th>';
		html += '<th class="dottedleft">All cases</th>';
		html += '<th class="dottedleft" colspan="3">Person data</th>';
		html += '<th class="dottedleft" colspan="2"></th>';
		html += '<th class="dottedleft" colspan="2"></th>';
		html += '</tr>';
		html += '<tr>';
		html += '<th>type</th>';
		html += '<th>name</th>';
		html += '<th class="dottedleft">type</th>';
		html += '<th>value</th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft">Personal number</th>';
		html += '<th>All first names</th>';
		html += '<th>Family name</th>';
		html += '<th class="dottedleft"></th>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th></th>';
		html += '</tr>';
		html += '</thead>';

		// Patient
		html += '<tbody>';
		html += '<tr>';
		if (!patientInfo)
		{
			html += '<td colspan="5"><span class="warning">No patient data is registered for this case</span></td>';
			html += '<td class="dottedleft">' + rf.asCaseList(caseNames, caseName) + '</td>';
			html += '<td class="dottedleft" colspan="5"></td>';
		}
		else
		{
			html += '<td>' + rf.warnIfMissing(patientInfo.subtype) + '</td>';
			html += '<td>' + rf.warnIfMissing(patientInfo.name) + '</td>';
			html += '<td class="dottedleft">Registration</td>';
			html += '<td>' + rf.warnIfMissing(rf.asDateTime(patientInfo.registrationDate)) + '</td>';
			html += '<td class="dottedleft">' + rf.warnIfMissing(patientInfo.gender) + '</td>';
			html += '<td class="dottedleft">' + rf.asCaseList(caseNames, caseName) + '</td>';
			html += '<td class="dottedleft">' + rf.warnIfMissing(patientInfo.personalNumber) + '</td>';
			html += '<td>' + rf.warnIfMissing(patientInfo.allFirstNames) + '</td>';
			html += '<td>' + rf.warnIfMissing(patientInfo.familyName) + '</td>';
			html += '<td class="dottedleft"></td>';
			html += '<td></td>';
			html += '<td class="dottedleft"></td>';
			html += '<td></td>';
		}
		html += '</tr>';
		html += '</tbody>';

		// Case
		var caseInfo = response.caseInfo;
		if (caseInfo && caseInfo.length > 0)
		{
			// Case Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr>';
			html += '<th colspan="2">Cases</th>';
			html += '<th class="dottedleft" colspan="2">Date</th>';
			html += '<th class="dottedleft" colspan="2">Consent</th>';
			html += '<th class="dottedleft" colspan="3">Other data</th>';
			html += '<th class="dottedleft" colspan="2">Flag</th>';
			html += '<th class="dottedleft" colspan="2"></th>';
			html += '</tr>';
			html += '<tr>';
			html += '<th>type</th>';
			html += '<th>name</th>';
			html += '<th class="dottedleft">type</th>';
			html += '<th>value</th>';
			html += '<th class="dottedleft">value</th>';
			html += '<th>date</th>';
			html += '<th class="dottedleft">Site</th>';
			html += '<th>Laterality</th>';
			html += '<th>Comment</th>';
			html += '<th class="dottedleft">value</th>';
			html += '<th>to set</th>';
			html += '<th class="dottedleft"></th>';
			html += '<th></th>';
			html += '</tr>';
			html += '</thead>';

			html += '<tbody>';

			var truncateAt = TRUNCATE_SIZE[Math.min(caseInfo.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < caseInfo.length; i++)
			{
				var c = caseInfo[i];
				var flag = c.flag ? c.flag : '-';
				var site = c.site;
				var comment = c.comment ? c.comment : '-';
				//var autoProcessing = s.autoProcessing ? s.autoProcessing : '-';
				html += '<tr>';
				html += '<td>Case</td>';
				html += '<td>' + c.name + '</td>';
				html += '<td class="dottedleft">Registration</td>';
				html += '<td>' + rf.warnIfMissing(rf.asDateTime(c.registrationDate)) + '</td>';
				html += '<td class="dottedleft">' + rf.asConsentAnyDate(c.consent) + '</td>';
				html += '<td>' + rf.warnIfMissing(rf.asDateTime(c.consentDate)) + '</td>';
				html += '<td class="dottedleft">' + site.name + '</td>';
				html += '<td>' + rf.warnIfMissing(c.laterality) + '</td>';
				html += '<td>' + comment + '</td>';
				html += '<td class="dottedleft">' + flag + '</td>';
				html += '<td>-</td>';
				html += '<td class="dottedleft"></td>';
				html += '<td></td>';
				html += '</tr>';
			}
			html += '</tbody>';
		}
		else
		{
			html += '<thead class="bg-filled-100">';
			html += '<tr id="case.name2">';
			html += '<th id="case-header2" colspan="2">Cases</th>';
			html += '<td colspan="11" style="text-align: left;">' + rf.asNoInfo('No case information has been registered') + '</td>';
			html += '</tr>';
			html += '</thead>';
		}

		// Blood
		var blood = response.blood;
		if (blood && blood.length > 0)
		{
			// Blood Sample Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr>';
			html += '<th colspan="2">Blood samples</th>';
			html += '<th class="dottedleft" colspan="2">Date</th>';
			html += '<th class="dottedleft" colspan="2">Consent</th>';
			html += '<th class="dottedleft" colspan="3">Other data</th>';
			html += '<th class="dottedleft" colspan="2">Flag</th>';
			html += '<th class="dottedleft" colspan="2"></th>';
			html += '</tr>';
			html += '<tr>';
			html += '<th>type</th>';
			html += '<th>name</th>';
			html += '<th class="dottedleft">type</th>';
			html += '<th>value</th>';
			html += '<th class="dottedleft">value</th>';
			html += '<th>date</th>';
			html += '<th class="dottedleft">Sample type</th>';
			html += '<th>RCC-ID</th>';
			html += '<th>Serum</th>';
			html += '<th class="dottedleft">value</th>';
			html += '<th>to set</th>';
			html += '<th class="dottedleft"></th>';
			html += '<th></th>';
			html += '</tr>';
			html += '</thead>';

			html += '<tbody>';
		
			var truncateAt = TRUNCATE_SIZE[Math.min(blood.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < blood.length; i++)
			{
				var b = blood[i];
				var flag = b.flag ? b.flag : '-';
				var bloodSample = b.bloodSample ? b.bloodSample : '-';
				var bloodRccidNumber = b.bloodRccidNumber ? b.bloodRccidNumber : '-';
				var bloodSerum = b.serum ? b.serum : '-';
				html += '<tr>';
				html += '<td>Blood</td>';
				html += '<td>' + b.name + '</td>';
				html += '<td class="dottedleft">Sampling</td>';
				html += '<td>' + rf.warnIfMissing(rf.asDateTime(b.samplingDate)) + '</td>';
				html += '<td class="dottedleft">' + rf.asConsentAnyDate(b.consent) + '</td>';
				html += '<td>' + rf.warnIfMissing(rf.asDateTime(b.consentDate)) + '</td>';
				html += '<td class="dottedleft">' + bloodSample + '</td>';
				html += '<td>' + bloodRccidNumber + '</td>';
				html += '<td>' + bloodSerum + '</td>';
				html += '<td class="dottedleft">' + flag + '</td>';
				html += '<td>-</td>';
				html += '<td class="dottedleft"></td>';
				html += '<td></td>';
				html += '</tr>';
			}
			html += '</tbody>';
		}
		else
		{
			// Blood Sample Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr id="blood.name2">';
			html += '<th id="blood-header2" colspan="2">Blood samples</th>';
			html += '<td colspan="11" style="text-align: left;">' + rf.asNoInfo('No blood information has been registered') + '</td>';
			html += '</tr>';
			html += '</thead>';
		}

		// Other sample items

		var specimen = response.specimen;
		var noSpecimen = response.noSpecimen;
		var histology = response.histology;
		var stained = response.stained;
		var lysate = response.lysate;
		var rna = response.rna;
		var rnaQc = response.rnaQc;
		var dna = response.dna;
		var ft = response.flowThrough;
		var mrna = response.mrna;
		var cdna = response.cdna;
		var lib = response.lib;
		var pooledLib = response.pooledlib;
		var bloodDna = response.bloodDna;

		// Count number of items to be included in lab protocol for destruction
		numItemsToBeDestroyed = 0;
		if (specimen && specimen.length > 0)
		{
			numItemsToBeDestroyed += specimen.length;
		}
		if (lysate && lysate.length > 0)
		{
			numItemsToBeDestroyed += lysate.length;
		}
		if (rna && rna.length > 0)
		{
			numItemsToBeDestroyed += rna.length;
		}
		if (dna && dna.length > 0)
		{
			numItemsToBeDestroyed += dna.length;
		}
		if (ft && ft.length > 0)
		{
			numItemsToBeDestroyed += ft.length;
		}
		if (bloodDna && bloodDna.length > 0)
		{
			numItemsToBeDestroyed += bloodDna.length;
		}

		// Check if any items exist
		var itemExists = false;
		if (specimen && specimen.length > 0)
		{
			itemExists = true;
		}
		if (noSpecimen && noSpecimen.length > 0)
		{
			itemExists = true;
		}
		if (histology && histology.length > 0)
		{
			itemExists = true;
		}
		if (lysate && lysate.length > 0)
		{
			itemExists = true;
		}
		if (rna && rna.length > 0)
		{
			itemExists = true;
		}
		if (dna && dna.length > 0)
		{
			itemExists = true;
		}
		if (ft && ft.length > 0)
		{
			itemExists = true;
		}
		if (mrna && mrna.length > 0)
		{
			itemExists = true;
		}
		if (cdna && cdna.length > 0)
		{
			itemExists = true;
		}
		if (lib && lib.length > 0)
		{
			itemExists = true;
		}
		if (pooledLib && specimen.length > 0)
		{
			itemExists = true;
		}
		if (bloodDna && bloodDna.length > 0)
		{
			itemExists = true;
		}

		if (itemExists)
		{
			// Sample Items Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr>';
			html += '<th colspan="2">Other items</th>';
			html += '<th class="dottedleft" colspan="2">Date</th>';
			html += '<th class="dottedleft" colspan="2">Other data</th>';
			html += '<th class="dottedleft" colspan="3">Storage</th>';
			html += '<th class="dottedleft" colspan="2">Flag</th>';
			html += '<th class="dottedleft" colspan="2">AutoProcessing</th>';
			html += '</tr>';
			html += '<tr>';
			html += '<th>type</th>';
			html += '<th>name</th>';
			html += '<th class="dottedleft">type</th>';
			html += '<th>value</th>';
			html += '<th class="dottedleft">name</th>';
			html += '<th>value</th>';
			html += '<th class="dottedleft" colspan="2">box/plate</th>';
			html += '<th>location</th>';
			html += '<th class="dottedleft">value</th>';
			html += '<th>to set</th>';
			html += '<th class="dottedleft">value</th>';
			html += '<th>to set</th>';
			html += '</tr>';
			html += '</thead>';

			html += '<tbody>';
			// Other sample item rows
			html += htmlTableRowsForItems(specimen, 'Specimen', 'specimen', 'Sampling', 'PAD', 'Storage box', false, false);
			html += htmlTableRowsForItems(noSpecimen, 'NoSpecimen', 'nospecimen', 'Sampling', 'PAD', 'NA', false, false);
			html += htmlTableRowsForItems(histology, 'Histology', 'histology', 'Partition', null, 'Paraffin block', true, true);
			html += htmlTableRowsForItems(stained, 'Stained', 'stained', null, null, 'Slide', false, false);
			html += htmlTableRowsForItems(lysate, 'Lysate', 'lysate', 'Lysis', null, 'Storage box', false, false);
			html += htmlTableRowsForItems(rna, 'RNA', 'rna', null, null, 'Storage box', true, true);
			html += htmlTableRowsForItems(rnaQc, 'RNA-QC', 'rnaqc', null, null, 'Plate', false, false);
			html += htmlTableRowsForItems(dna, 'DNA', 'dna', null, null, 'Storage box', false, false);
			html += htmlTableRowsForItems(ft, 'FlowThrough', 'flowthrough', null, null, 'Storage box', false, false);
			html += htmlTableRowsForItems(mrna, 'mRNA', 'mrna', null, null, 'Work plate', false, false);
			html += htmlTableRowsForItems(cdna, 'cDNA', 'cdna', null, null, 'Work plate', false, false);
			html += htmlTableRowsForItems(lib, 'Library', 'library', null, null, 'Library plate', false, false);
			html += htmlTableRowsForItems(pooledLib, 'Pooled Library', 'pooledlibrary', null, null, null, false, false);
			html += htmlTableRowsForItems(bloodDna, 'BloodDNA', 'bloodDna', null, null, 'Storage box', false, false);
			html += '</tbody>';
		}
		else
		{
			// Other Items Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr id="other.name2">';
			html += '<th id="other-header2" colspan="2">Other items</th>';
			html += '<td colspan="11" style="text-align:left;">' + rf.asNoInfo('No other sample item information has been registered') + '</td>';
			html += '</tr>';
			html += '</thead>';
		}

		// Sequencing runs
		var sr = response.sequencingRun;
		if (sr && sr.length > 0)
		{
			// Sequencing runs Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr>';
			html += '<th colspan="2">Sequencing runs</th>';
			html += '<th class="dottedleft" colspan="2">Date</th>';
			html += '<th class="dottedleft" colspan="2">Flow cell</th>';
			html += '<th class="dottedleft" colspan="3">Other data</th>';
			html += '<th class="dottedleft" colspan="2">Flag NA</th>';
			html += '<th class="dottedleft" colspan="2">AutoProcessing</th>';
			html += '</tr>';
			html += '<tr>';
			html += '<th colspan="2">name</th>';
			html += '<th class="dottedleft">start</th>';
			html += '<th>end</th>';
			html += '<th class="dottedleft">name</th>';
			html += '<th>cluster date</th>';
			html += '<th class="dottedleft">Case</th>';
			html += '<th>Result</th>';
			html += '<th>Comment</th>';
			html += '<th class="dottedleft"></th>';
			html += '<th></th>';
			html += '<th class="dottedleft">value</th>';
			html += '<th>to set</th>';
			html += '</tr>';
			html += '</thead>';

			html += '<tbody class="bottomborder">';
		
			var truncateAt = TRUNCATE_SIZE[Math.min(sr.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < sr.length; i++)
			{
				var seqrun = sr[i];
				var autoProcessing = seqrun.autoProcessing ? seqrun.autoProcessing : '-';
				var flowCell = seqrun.flowCell;
				var seqrunCase = seqrun.caseName;
				var seqrunResult = seqrun.result;
				html += '<tr>';
				html += '<td colspan="2">' + seqrun.name + '</td>';
				html += '<td class="dottedleft">' + rf.warnIfMissing(rf.asDateTime(seqrun.startDate)) + '</td>';
				html += '<td>' + rf.warnIfMissing(rf.asDateTime(seqrun.endDate)) + '</td>';
				html += '<td class="dottedleft">' + flowCell.name + '</td>';
				html += '<td>' + rf.warnIfMissing(rf.asDateTime(flowCell.clusterDate)) + '</td>';
				html += '<td class="dottedleft">' + seqrun.caseName + '</td>';
				html += '<td>' + seqrun.result + '</td>';
				html += '<td>' + seqrun.comment + '</td>';
				html += '<td class="dottedleft">-</td>';
				html += '<td>-</td>';
				html += '<td class="dottedleft">' + autoProcessing + '</td>';
				html += '<td>-</td>';
				html += '</tr>';
			}
			html += '</tbody>';
		}
		else
		{
			// Sequencing Runs Header row
			html += '<thead class="bg-filled-100">';
			html += '<tr id="seqruns.name2">';
			html += '<th id="seqruns-header2" colspan="2">Sequencing runs</th>';
			html += '<td colspan="11" style="text-align: left;">' + rf.asNoInfo('No sequencing run information has been registered') + '</td>';
			html += '</tr>';
			html += '</thead>';
		}

		html += '</table>';
		Doc.element('retractTable').innerHTML = html;

		// Initialize biosource type radio buttons
		
		var useRetract = !patientInfo || patientInfo.name.indexOf('RetroNo') == -1;
		if (useRetract)
		{
			// Use "Retract" for "PAT*" and "Retract*" biosources
			rf.enableNewBiosourceTypeOption('newbiosourcetype.retract', true);
			rf.enableNewBiosourceTypeOption('newbiosourcetype.retrono', false);
		}
		else
		{
			// Use "RetroNo" for "No*" and "Not asked*" biosources
			rf.enableNewBiosourceTypeOption('newbiosourcetype.retract', false);
			rf.enableNewBiosourceTypeOption('newbiosourcetype.retrono', true);
		}

		Wizard.setCurrentStep(2);

		Doc.show('step-1');
		Doc.show('gocancel');
		Doc.show('goregister');
		Doc.show('gocreate');
	}


	function htmlTableRowsForItems(
		itemJsonArr,
		itemType,
		itemTypeVarName,
		dateType,
		otherDataType,
		storageType,
		setRetracted,
		setAutoProcessingDisabled)
	{
		// Table item row
		var html = '';
		if (itemJsonArr && itemJsonArr.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(itemJsonArr.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < itemJsonArr.length; i++)
			{
				var item = itemJsonArr[i];
				var flag = item.flag ? item.flag : '-';
				var autoProcessing = item.autoProcessing ? item.autoProcessing : '-';
				var itemDate = '-';
				if (dateType)
				{
					if (dateType == 'Sampling')
					{
						itemDate = rf.warnIfMissing(rf.asDateTime(item.samplingDate));
					}
					else if (dateType == 'Partition')
					{
						itemDate = rf.warnIfMissing(rf.asDateTime(item.partitionDate));
					}
					else if (dateType == 'Lysis')
					{
						itemDate = rf.warnIfMissing(rf.asDateTime(item.lysisDate));
					}
				}
				else
				{
					dateType = '-';
				}
				var otherDateName = '-';
				var otherDateValue = '-';
				var otherDataName = '-';
				var otherDataValue = '-';
				if (otherDataType)
				{
					if (otherDataType == 'PAD')
					{
						otherDataName = 'PAD';
						otherDataValue = rf.warnIfMissing(item.pad);
					}
				}
				var storageName = '-';
				var storageId = '-';
				var storagePosition = '-';
				var storageCoordinate = '-';
				if (storageType)
				{
					if (storageType == 'NA')
					{
						storageName = 'NA';
						storageId = 'NA';
						storagePosition = 'NA';
					}
					else if (item.bioWell)
					{
						storageName = storageType;
						storageId = rf.asBioPlateName(item.bioWell);
						storageCoordinate = item.bioWell ? item.bioWell.location : '-';
						storagePosition = rf.asPureBioPlateLocation(item.bioWell);
					}
				}
				html += '<tr>';
				html += '<td>' + itemType + '</td>';
				html += '<td>' + item.name + '</td>';
				html += '<td class="dottedleft">' + dateType + '</td>';
				html += '<td>' + itemDate + '</td>';
				html += '<td class="dottedleft">' + otherDataName + '</td>';
				html += '<td>' + otherDataValue + '</td>';
				html += '<td class="dottedleft">' + storageId + '</td>';
				html += '<td>' + storageCoordinate + '</td>';
				html += '<td>' + storagePosition + '</td>';
				html += '<td class="dottedleft">' + flag + '</td>';
				if (setRetracted)
				{
					var checkboxName = itemTypeVarName + '.retracted.' + item.id;
					//html += '<td><input type="checkbox" name="' + checkboxName + '" checked>Retracted ' + checkboxName + '</td>';
					html += '<td><input type="checkbox" name="' + checkboxName + '" checked>Retracted</td>';
				}
				else
				{
					var checkboxName = itemTypeVarName + '.retracted.' + item.id;
					//html += '<td>- ' + checkboxName + '</td>';
					html += '<td>-</td>';
				}
				html += '<td class="dottedleft">' + autoProcessing + '</td>';
				if (setAutoProcessingDisabled)
				{
					var checkboxName = itemTypeVarName + '.autoProcessingDisable.' + item.id;
					//html += '<td><input type="checkbox" name="' + checkboxName + '" checked>Disable ' + checkboxName + '</td>';
					html += '<td><input type="checkbox" name="' + checkboxName + '" checked>Disable</td>';
				}
				else
				{
					var checkboxName = itemTypeVarName + '.autoProcessingDisable.' + item.id;
					//html += '<td>- ' + checkboxName + '</td>';
					html += '<td>-</td>';
				}
				html += '</tr>';
			}
		}
		return html;
	}




	function checkBoxValues(selectedSamples)
	{
		var inputs = document.getElementsByTagName("input");
		var checkboxes = [];
		for (var i = 0; i < inputs.length; i++)
		{
			if (inputs[i].type == "checkbox")
			{
				// Require that checkbox name contains a dot '.' to be included in list
				if (inputs[i].name.indexOf(".") > -1)
				{
					checkboxes.push(inputs[i]);
					if (inputs[i].checked)
					{
						selectedSamples.push(inputs[i].name);
					}
					else
					{
						selectedSamples.push(inputs[i].name + '_unchecked');
					}
				}
			}
		}
		return selectedSamples;
	}

	rf.asCaseList = function(allCases, mainCase)
	{
		var html = '';
		for (var i = 0; i < allCases.length; i++)
		{
			var cse = allCases[i];
			if (html != '') html += ', ';
			if (cse != mainCase)
			{
				//html += '<span class="link linked-case" data-case-name="'+cse+'" title="Show summary of case #'+cse+'">'+cse+'</span>';
				html += cse;
			}
			else
			{
				html += '<b>' + cse + '</b>';
			}
		}
		return allCases.length > 0 ? html : null;
	}

	
	// Format value as a date
	rf.asDate = function(value)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = value.substr(0, 4) + '-' + value.substr(4, 2) + '-' + value.substr(6, 2);
		}
		return value;
	}

	// Format as date+time value
	rf.asDateTime = function(value, compareToDate)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = rf.asDate(value);
		}
		else if (value.length == 13)
		{
			// If the compareToDate is the same day as the 'value' date, skip the date and replace with white-space
			if (compareToDate && value.substr(0, 8) == compareToDate.substr(0, 8))
			{
				value = '<span class="invisible">'+rf.asDate(value.substr(0, 8)) + '</span> ' + value.substr(9, 2) + ':'+value.substr(11, 2)
			}
			else
			{
				value = rf.asDate(value.substr(0, 8)) + ' ' + value.substr(9, 2) + ':'+value.substr(11, 2);
			}
		}
		return value;
	}

	// Style the message as a warning message
	rf.asWarning = function(message)
	{
		message = '<span class="warning">' + message + '</span>';
		return message;
	}

	// Generate a 'Missing' warning if the value is missing
	rf.warnIfMissing = function(value)
	{
		var message = value ? value : rf.asWarning('Missing');
		return message;
	}
	
	// Style the message as a 'no information' message
	rf.asNoInfo = function(message)
	{
		message = '<span class="no-info">' + message + '</span>';
		return message;
	}
	
	rf.asConsentAnyDate = function(consent)
	{
		var warn = !consent;
		var message;
		if (!consent)
		{
			message = 'Missing';
		}
		else
		{
			message = consent;
			
		}
		return warn ? rf.asWarning(message) : message;
	}
	
	// A biowell as plate name
	rf.asBioPlateName = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		if (!plate) return '';
		var text = plate.name;
		return text;
	}

	// A biowell as the location
	rf.asPureBioPlateLocation = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		var text = '';
		if (plate.storage)
		{
			var storage = plate.storage;
			var tmp = [];
			if (storage.name) tmp[tmp.length] = storage.name;
			if (storage.section) tmp[tmp.length] = 'section: ' +storage.section;
			if (storage.tray) tmp[tmp.length] = 'tray: ' +storage.tray;
			if (storage.position) tmp[tmp.length] = 'position: ' +storage.position;

			//text = '<span class="more-info" title="' + tmp.join('; ')+'">' + text + ' </span>';
			text = text + ' ' + tmp.join('; ');
		}
		return text;
	}

	rf.truncate = function(value, maxLength)
	{
		if (!value) return value;
		if (maxLength > 2 && value.length > maxLength) 
		{
			var tmp = '<span class="truncated" title="'+value+'">'+value.substring(0, maxLength-2) + '...</span>';
			value = tmp;
		}
		return value;
	}

	rf.goPrint = function()
	{
		var caseName = Data.get('page-data', 'case-name');
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.<br>';
		printNote += ' You may have to <i>scale down</i> to fit everything on the width of the page.';
		Reggie.openPrintWindow('all-content', 'Case summary - ' + Strings.encodeTags(caseName), 'portrait', printNote, '../', 'case_summary.css');
	}

	rf.submit = function()
	{
		var frm = document.forms['reggie'];

		var retractionInfo = {};
		retractionInfo.caseName = frm.caseName.value;
		var selectedSamples = [];
		// Other sample items
		selectedSamples = checkBoxValues(selectedSamples);
		retractionInfo.selectedSamples = selectedSamples;

		// Force reset when unchecked flag
		var forceResetWhenUnchecked = false;
		var forceResetWhenUncheckedCB = document.getElementById("forceResetWhenUnchecked");
		if (forceResetWhenUncheckedCB.checked)
		{
			forceResetWhenUnchecked = true;
		}
		retractionInfo.forceResetWhenUnchecked = forceResetWhenUnchecked;

		retractionInfo.bioSourceType = Forms.getCheckedRadio(frm.newbiosourcetype).value;

		var submitInfo = {};
		submitInfo.retractionInfo = retractionInfo;
		
		var url = '../Retraction.servlet?ID=' + App.getSessionId();
		url += '&cmd=RegisterRetraction';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, rf.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	
	rf.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}


	rf.createProtocol = function()
	{
		var frm = document.forms['reggie'];
		
		var url = 'retraction_protocol2.jsp?ID='+App.getSessionId();
		url += '&caseName='+encodeURIComponent(frm.caseName.value);

		window.open(url);
	}


	rf.enableNewBiosourceTypeOption = function(option, checkIt)
	{
		option = Doc.element(option); // The radio button
		option.disabled = false;
		if (checkIt) option.checked = true;
		
		var label = Doc.element(option.id + '.label'); // The label
		Doc.removeClass(label, 'disabled');
	}


	return rf;
}();

Doc.onLoad(RetractionForm.initPage);

