var LysateReExtract = function()
{
	var extract = {};
	var debug = 0;
	
	var lysatesIsValid = false;
	var selectedLysates;
	
	// Page initialization
	extract.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', extract.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', extract.initializeStep2);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('goprint', extract.printPickList);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', extract.submit);

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLysatesForPicking';		
		Wizard.showLoadingAnimation('Loading Lysates selected for re-extraction...');
		Wizard.asyncJsonRequest(url, extract.initializeStep1);
	}
	
	extract.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		var lysates = response.lysates;
		
		var lysateList = frm.lysates;
		if (lysates.length > 0)
		{
			for (var i=0; i < lysates.length; i++)
			{
				var lysate = lysates[i];
				var isYellow = lysate.specimen.YellowLabel != null;
				var name = (i+1) + ': ' + Strings.encodeTags(lysate.name);
				if (lysate.bioWell)
				{
					name += ' -- ' + Strings.encodeTags(lysate.bioWell.bioPlate.name + ' (' + lysate.bioWell.location+')');
				}

				var option = new Option(name, lysate.id, true, true);
				if (isYellow) option.className = 'yellow';
				option.lysate = lysate;
				lysateList.options[lysateList.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No Lysates available for re-extraction.');
			return;
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	extract.printPickList = function()
	{
		var frm = document.forms['reggie'];
		var selected = [];
		for (var i = 0; i < frm.lysates.length; i++)
		{
			if (frm.lysates[i].selected) 
			{
				selected[selected.length] = frm.lysates[i].value;
			}
		}
		
		var url = 'lysate_picklist.jsp?ID='+App.getSessionId();
		url += '&lysates='+selected.join(',');
		window.open(url, '_blank');
		Wizard.hideGoNextConfirmation();
	}

	extract.validateStep1 = function(event)
	{
		var frm = document.forms['reggie'];
		var lysateList = frm.lysates;
		
		selectedLysates = [];
		for (var i=0; i < lysateList.length; i++)
		{
			if (lysateList[i].selected)
			{
				selectedLysates[selectedLysates.length] = lysateList[i].lysate;
			}
		}
			
		if (selectedLysates.length == 0)
		{
			Wizard.setInputStatus('lysates', 'invalid', 'Not selected', 'invalid');
			event.preventDefault();
		}
		else
		{
			Wizard.setInputStatus('lysates', 'valid');
		}
	}
	
	extract.initializeStep2 = function()
	{
		
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetNewLysateBoxPositions&numLysates='+selectedLysates.length;		
		Wizard.showLoadingAnimation('Finding new box positions for lysates...');
		Wizard.asyncJsonRequest(url, extract.newBoxPositionsLoaded);

	}
	
	extract.newBoxPositionsLoaded = function(response)
	{
		var newWells = response.wells;
		if (newWells.length < selectedLysates.length)
		{
			var msg;
			if (!response.lastExtraLys)
			{
				msg = 'Could not find any extra storage boxes for';
				msg += ' Lysate, RNA, DNA and FlowThrough.';
				msg += ' Please prepare and register extra storage boxes before using this wizard.';
			}
			else
			{
				var msg = 'Could not find any empty space in <i>' + Strings.encodeTags(response.lastExtraLys) + '</i>';
				msg += ' and related storage boxes for Lysate, RNA, DNA and FlowThrough.';
				msg += ' Please prepare and register extra storage boxes before using this wizard.';
			}
			Wizard.setFatalError(msg);
			return;
		}
		
		var html = '';
		var homeUrl = Data.get('page-data', 'home-url');
		var yellowImg = '<img src="'+homeUrl+'/images/yellow-label.png">';
		var numWithLabel = 0;
		for (var i=0; i < selectedLysates.length; i++)
		{
			var lysate = selectedLysates[i];
			var well = lysate.bioWell;
			var plate = well.bioPlate;
			var storage = plate.storage;
			var isYellow = lysate.specimen && lysate.specimen.YellowLabel != null;
			var newWell = newWells[i];
			lysate.newWell = newWell;
			if (lysate.label) numWithLabel++;
			
			var freezerLocation = [];
			if (storage)
			{
				if (storage.section) freezerLocation[freezerLocation.length] = 'Section '+storage.section;
				if (storage.tray) freezerLocation[freezerLocation.length] = 'Tray '+storage.tray;
				if (storage.position) freezerLocation[freezerLocation.length] = 'Position '+storage.position;
			}

			var img = isYellow ? yellowImg : '';
			html += '<tr class="highlight'+(isYellow ? ' yellow-specimen' : '')+'">';
			html += '<td class="lysate if-yellow">'+img+Strings.encodeTags(lysate.name)+'</td>';
			html += '<td class="label">'+Strings.encodeTags(lysate.label)+'</td>';
			html += '<td class="location">';
			if (storage)
			{
				html += Strings.encodeTags(storage.name)+'<br>'+Strings.encodeTags(freezerLocation.join(', '));
			}
			html += '</td>';
			html += '<td class="box">'+Strings.encodeTags(plate.name+' '+well.location)+'</td>';
			html += '<td class="newbox">'+Strings.encodeTags(newWell.bioPlate.name+' '+newWell.location)+'</td>';
			html += '<td class="remain">'+ Numbers.formatNumber(lysate.remainingQuantity, 0, 'µl') + '</td>';
			html += '<td class="remarks"></td>';
			html += '</tr>'
		}
		
		Doc.element('lysateList').innerHTML = html;
		if (numWithLabel > 0) 
		{
			Doc.removeClass('protocol-table', 'nolabels');
		}
		
		Wizard.setCurrentStep(2);
		Doc.show('goregister');
		Doc.show('goprint');
		Doc.show('gocancel');
		Wizard.showGoNextConfirmation(true, 'Print the lab protocol before continuing with the registration!');
	}
	
	extract.printPickList = function()
	{
		Wizard.hideGoNextConfirmation();
		Doc.removeClass('goregister', 'disabled');
		var reportName = 'Pick-list protocol for lysate re-extraction';
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.';
		Reggie.openPrintWindow('full-protocol', reportName, 'portrait', printNote, '../', 'lysate_picklist.css');
	}

	extract.submit = function()
	{
		var submitInfo = {};
		var lysates = [];
		for (var i = 0; i < selectedLysates.length; i++)
		{
			var lysate = selectedLysates[i];
			lysates[i] = { 'id': lysate.id, 'newWell': lysate.newWell.id };
		}
		submitInfo.lysates = lysates;
		
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterLysatesPickedForReextraction';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, extract.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	extract.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return extract;
}();

Doc.onLoad(LysateReExtract.initPage);

