var PausedSpecimen = function()
{
	var paused = {};
	var debug = 0;
	
	var specimen;
	var sites;
	
	// Page initialization
	paused.initPage = function()
	{
		// Step 1
		Events.addEventHandler('nameFilter', 'change', paused.filterOnChange);
		Events.addEventHandler('statusFilter', 'change', paused.filterOnChange);
		Events.addEventHandler('siteFilter', 'change', paused.filterOnChange);
		Events.addEventHandler('samplingDateFilter', 'change', paused.filterOnChange);
		Events.addEventHandler('arrivalDateFilter', 'change', paused.filterOnChange);
		Events.addEventHandler('storageBoxFilter', 'change', paused.filterOnChange);
		Buttons.addClickHandler('btnExtractSpecimen', paused.extractSpecimen);
		Buttons.addClickHandler('btnPauseSpecimen', paused.pauseSpecimen);
		Buttons.addClickHandler('btnPauseSettings', paused.openSettings);
		Buttons.addClickHandler('check_uncheck', paused.selectAll);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetPausedSpecimenTubes';
		Wizard.showLoadingAnimation('Loading paused specimen...');
		Wizard.asyncJsonRequest(url, paused.onSpecimenLoaded);
	}
	
	paused.onSpecimenLoaded = function(response)
	{
		specimen = response.specimenTubes;
		sites = response.sites;

		var frm = document.forms['reggie'];
		var siteCount = [];
		var pausedSites = [];
		for (var siteNo = 0; siteNo < sites.length; siteNo++)
		{
			var site = sites[siteNo];
			if (site.paused) pausedSites[pausedSites.length] = site.name;
			siteCount[site.prefix] = 0;
		}
		Doc.element('pausedInfo').innerHTML = pausedSites.length > 0 ? 
			'<b>Paused sites</b>: ' + pausedSites.join(', ') : '<b>No sites are paused</b>';
		
		var html = '';
		for (var spNo = 0; spNo < specimen.length; spNo++)
		{
			var sp = specimen[spNo];
			sp.include = true;
			siteCount[sp.site.prefix]++;
			var rowClass = 'highlight';
			rowClass += spNo % 2 == 0 ? ' oddrow' : ' evenrow';
			if (sp.YellowLabel != null) rowClass += ' yellow-specimen';
			
			html += '<tr class="' + rowClass + '" id="specimen.'+sp.id + '">';
			html += '<td><span id="index.'+sp.id+'">'+(spNo+1)+'</span> <input type="checkbox" name="sp.'+sp.id+'"></td>';
			html += '<td class="dottedleft if-yellow"><span class="link linked-item" data-item-type="SAMPLE" data-item-id="'+sp.id+'">'+Strings.encodeTags(sp.name)+'</span>';
			html += '<img class="link case-summary" data-name="'+Strings.encodeTags(sp.name)+'" src="../images/case_summary.png"></td>';
			html += '<td>'+(sp.AutoProcessing == 'Disable' ? 'Paused' : 'Scheduled') + '</td>';
			html += '<td>'+Strings.encodeTags(sp.site.name) + '</td>';
			html += '<td>'+Reggie.reformatDate(sp.SamplingDate)+'</td>';
			html += '<td>'+Reggie.reformatDate(sp.ArrivalDate)+'</td>';
			html += '<td>'+Strings.encodeTags(sp.bioWell.bioPlate.name + ' ' + sp.bioWell.location)+'</td>';
			html += '<td style="width: auto;"></td>';
			html += '</tr>';
		}
		
		Doc.element('specimen-list').innerHTML = html;
	
		frm.siteFilter.length = 0;
		frm.siteFilter[0] = new Option(' ', '');
		for (var siteNo = 0; siteNo < sites.length; siteNo++)
		{
			var site = sites[siteNo];
			frm.siteFilter[frm.siteFilter.length] = new Option(site.name + ' ('+siteCount[site.prefix]+')', site.prefix);
		}
		
		paused.filterOnChange();
		
		if (submitMessages)
		{
			Wizard.showFinalMessage(submitMessages);
			submitMessages = null;
			setTimeout(Wizard.showFinalMessage, 10000, []); // Hide after 10 seconds
		}
		
		Doc.show('step-1');

		var cs = document.getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', Reggie.openCaseSummaryPopupOnEvent);
		}
		
		var linkedItems = document.getElementsByClassName('linked-item');
		for (var itemNo = 0; itemNo < linkedItems.length; itemNo++)
		{
			Events.addEventHandler(linkedItems[itemNo], 'click', Items.itemOnClick);
		}
	}

	paused.selectAll = function()
	{
		var frm = document.forms['reggie'];
		var firstChecked = null;
		for (var spNo = 0; spNo < specimen.length; spNo++)
		{
			var sp = specimen[spNo];
			if (sp.include)
			{
				if (firstChecked == null)
				{
					firstChecked = frm['sp.'+sp.id].checked;
				}
				frm['sp.'+sp.id].checked = !firstChecked;
			}
		}
	}
	
	paused.filterOnChange = function()
	{
		var frm = document.forms['reggie'];

		var filter = new Filter();
		filter.addContainsCriteria(ItemProperty.Name, frm.nameFilter.value);
		filter.addTextCriteria(ItemProperty.AutoProcessing, frm.statusFilter.value);
		filter.addTextCriteria(ItemProperty.Site, frm.siteFilter.value);
		filter.addDateCriteria(ItemProperty.SamplingDate, frm.samplingDateFilter.value);
		filter.addDateCriteria(ItemProperty.ArrivalDate, frm.arrivalDateFilter.value);
		filter.addContainsCriteria(ItemProperty.StorageBox, frm.storageBoxFilter.value);

		var index = 0;
		var numPaused = 0;
		for (var spNo = 0; spNo < specimen.length; spNo++)
		{
			var sp = specimen[spNo];
			sp.include = filter.check(sp);
			var tr = Doc.element('specimen.'+sp.id);
			if (sp.include)
			{
				index++;
				var evenRow = index % 2 == 0;
				Doc.addOrRemoveClass(tr, 'oddrow', !evenRow);
				Doc.addOrRemoveClass(tr, 'evenrow', evenRow);
				Doc.element('index.'+sp.id).innerHTML = index;
				Doc.show(tr);
				if (sp.AutoProcessing == 'Disable') numPaused++;
			}
			else
			{
				Doc.hide(tr);
			}
		}
		
		Doc.showHide('specimen-list', index != 0);
		Doc.showHide('no-match', index==0);
		Doc.addOrRemoveClass('btnExtractSpecimen', 'disabled', numPaused==0);
		Doc.addOrRemoveClass('btnPauseSpecimen', 'disabled', index==numPaused);
		
		Doc.element('num-matching').innerHTML = index;
	}
	
	paused.checkSelected = function(noNotification)
	{
		var frm = document.forms['reggie'];
		var selected = [];
		
		for (var spNo = 0; spNo < specimen.length; spNo++)
		{
			var sp = specimen[spNo];
			if (sp.include && frm['sp.'+sp.id].checked)
			{
				selected[selected.length] = sp.id;
			}
		}
		
		if (selected.length == 0 && !noNotification)
		{
			Forms.showNotification('check_uncheck', 'Please select at least one specimen from the list', null, 'pointer-below');
		}
		
		return selected;
	}

	paused.extractSpecimen = function()
	{
		var selected = paused.checkSelected();
		if (selected.length == 0) return;

		var submitInfo = {};
		submitInfo.specimen = selected;
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=SchedulePausedSpecimenForExtraction';
		Wizard.showLoadingAnimation('Registering selected specimen for extraction...');
		Wizard.asyncJsonRequest(url, paused.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	paused.pauseSpecimen = function()
	{
		var selected = paused.checkSelected();
		if (selected.length == 0) return;

		var submitInfo = {};
		submitInfo.specimen = selected;
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterPausedSpecimen';
		Wizard.showLoadingAnimation('Registering selected specimen as paused...');
		Wizard.asyncJsonRequest(url, paused.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	paused.openSettings = function()
	{
		var url = 'pause-settings.jsp?ID='+App.getSessionId();
		Dialogs.openPopup(url, 'PauseSettings', 500, 350);
	}
	
	var submitMessages;
	// Re-load the updated information
	paused.submissionResults = function(response)
	{
		submitMessages = response.messages;
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetPausedSpecimenTubes';
		Wizard.showLoadingAnimation('Loading paused specimen...');
		Wizard.asyncJsonRequest(url, paused.onSpecimenLoaded);
	}
	
	return paused;
}();

Doc.onLoad(PausedSpecimen.initPage);


/**
	Holds one or more filter criterias. Use the 'add*' methods
	to add criterias to the filter, and the 'check(specimen)' method
	to evaluate all criterias against a single specimen item.
*/
var Filter = function()
{
	this.criterias = [];
}

/**
	Adds a text criteria comparing the given property to
	a static value (f2).
*/
Filter.prototype.addTextCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.eq;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Adds a text criteria checking if the given item property to
	contains a static value (f2).
*/
Filter.prototype.addContainsCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.contains;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Adds a temporal criteria for comparing dates. The 'f2' value
	may start with either '<' or '>' to compare with earlier/later
	than operators. In all cases, the 'f2' value is stripped from '-'
	characters.
*/
Filter.prototype.addDateCriteria = function(itemProp, f2)
{
	if (f2 != '')
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			var firstChar = f2.charAt(0);
			if (firstChar == '<' || firstChar == '>') f2 = f2.substring(1);
			f2 = f2.replace(/\-/g, '');
			cmp = Comparator.get(firstChar, Comparator.startsWith);
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}


/**
	Evaluate all criterias. Return true if all pass, false otherwise.
*/
Filter.prototype.check = function(sp)
{
	for (var i = 0; i < this.criterias.length; i++)
	{
		if (!this.criterias[i](sp)) return false;
	}
	return true;
}

/**
	Functions for extracting various properties from
	specimen items. Each method accepts a single specimen as a parameter
	and return a value from the specimen item. The methods
	are used as parameters to Filter.add*Criteria() methods.
*/
var ItemProperty = function()
{
	var p = {};
	
	p.Name = function(item)
	{
		return item.name;
	}
	
	p.AutoProcessing = function(item)
	{
		return item.AutoProcessing;
	}
	
	// Get the item.site.prefix value
	p.Site = function(item)
	{
		return item.site.prefix;
	}
	
	p.ArrivalDate = function(item)
	{
		return item.ArrivalDate;
	}

	p.SamplingDate = function(item)
	{
		return item.SamplingDate;
	}

	p.StorageBox = function(item)
	{
		return item.bioWell.bioPlate.name + ' ' + item.bioWell.location;
	}
	
	return p;
}();

/**
	Comparator functions for comparing two values.
*/
var Comparator = function()
{
	var cmp = {};
	
	/**
		Return a comparator based on the symbol
		which can be either '>' or '<' to return
		Comparator.gt or Comparator.lt. Otherwise
		the given default or Comparator.eq is returned.
	*/
	cmp.get = function(symbol, defaultCmp)
	{
		var c;
		if (symbol == '<')
		{
			c = cmp.lt;
		}
		else if (symbol == '>')
		{
			c = cmp.gt;
		}
		else
		{
			c = defaultCmp || cmp.eq;
		}
		return c;
	}
	
	// Compare if two values are equal
	cmp.eq = function(v1, v2)
	{
		return v1 == v2;
	}
	
	// Compare if two values are different
	cmp.neq = function(v1, v2)
	{
		return v1 != v2;
	}
	
	// Check if v2 is contained in v1
	cmp.contains = function(v1, v2)
	{
		return v1 != null && v2 != null && v1.indexOf(v2) >= 0;
	}
	
	// Compare if v1 starts with the same string as v2
	cmp.startsWith = function(v1, v2)
	{
		return v1 != null && v1.indexOf(v2) == 0;
	}

	// Compare if two values are appoximately equals (difference is not more than 0.05)
	cmp.approx = function(v1, v2)
	{
		return Math.abs(v2 - v1) < 0.05;
	}
	
	// Compare if v1 is greater than or equal to v2
	cmp.gt = function(v1, v2)
	{
		return v1 >= v2;
	}
	
	// Compare if v1 is less than or equal to v2
	cmp.lt = function(v1, v2)
	{
		return v1 <= v2;
	}
	
	return cmp;
}();

