var RnaQcImport = function()
{
	var rnaqc = {};
	var bioplate;
	var THIN_SPACE = '&#x2005;';
	var debug = 0;
	
	// Page initialization
	rnaqc.initPage = function()
	{
		// Step 1
		Wizard.initFileSelectionField('csv');
		Wizard.initFileSelectionField('gxdstd');
		Wizard.initFileSelectionField('gxdhi');
		Wizard.initFileSelectionField('pdf');
		Events.addEventHandler('csv', 'change', rnaqc.preValidateCsvFile);
		Events.addEventHandler('qcRunDate', 'change', Wizard.validateDate);
		Events.addEventHandler('step-1', 'wizard-validate', rnaqc.validateStep1);
		Events.addEventHandler('bioplate.name', 'click', rnaqc.bioplateOnClick);

		Events.addEventHandler('extraAliquot.Yes', 'click', rnaqc.extraAliquotOnClick);
		Events.addEventHandler('extraAliquot.No', 'click', rnaqc.extraAliquotOnClick);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', rnaqc.submit);

		var frm = document.forms['reggie'];
		frm.csv.focus();
		Doc.show('step-1');
		Doc.show('goregister');
	}
	
	rnaqc.preValidateCsvFile = function()
	{
		var frm = document.forms['reggie'];
		var csv = frm.csv.value;
		if (!csv) return;
		
		Wizard.setInputStatus('bioplate');
		Doc.element('bioplate.name').innerHTML = '<i>(no file loaded)</i>';
		Doc.element('extraAliquot.No').click();
		Doc.element('extraAliquot.Yes').disabled = true;
		Data.set('bioplate.name', 'item-id', 0);
		Doc.removeClass('bioplate.name', 'link');
		Wizard.setInputStatus('csv', 'checking', 'Checking...');
		
		var url = '../RnaQc.servlet?ID='+App.getSessionId();
		url += '&cmd=PreValidateCaliperResults&csv='+encodeURIComponent(csv);
		Wizard.asyncJsonRequest(url, rnaqc.onCsvValidated, 'POST');
	}

	rnaqc.onCsvValidated = function(response)
	{
		var frm = document.forms['reggie'];
		Wizard.setInputStatus('csv');
		Wizard.showFinalMessage(response.messages);
		Wizard.setNoConfirmOnFirstStep(false);

		bioplate = null;
		if (!response.valid)
		{
			Wizard.setInputStatus('csv', 'invalid');
			Data.set('csv', 'valid', 0);
			return;
		}

		Wizard.setInputStatus('csv', response.warnings > 0 ? 'warning' : 'valid');
		Data.set('csv', 'valid', 1);
		
		bioplate = response.plate;
		
		Doc.element('bioplate.name').innerHTML = Strings.encodeTags(bioplate.name);
		Data.set('bioplate.name', 'item-id', bioplate.id);
		Doc.addClass('bioplate.name', 'link');
		frm.comments.value = bioplate.comments;
		Doc.element('extraAliquot.Yes').disabled = false;
		
		Wizard.setInputStatus('bioplate', 'valid');
		if (bioplate.QCRunDate)
		{
			frm.qcRunDate.value = bioplate.QCRunDate;
			Events.sendChangeEvent('qcRunDate');
			Wizard.setInputStatus('bioplate', 'warning', 'Data has already been imported to this plate');
		}
		else
		{
			// Try to get date from the filename
			var dates = frm.csv.value.match(/\d\d\d\d-\d\d-\d\d/);
			if (dates && dates.length >= 1)
			{
				var date = dates[0];
				frm.qcRunDate.value = date.substr(0, 4)+date.substr(5, 2)+date.substr(8, 2);
				Events.sendChangeEvent('qcRunDate');
			}
		}
		
		if (bioplate.QCOperator)
		{
			frm.qcOperator.value = bioplate.QCOperator;	
		}
		
		var html = rnaqc.buildExtraAliquotPlate(bioplate);
		Doc.element('extraAliquotPlate').innerHTML = html;
		
		var wells = Doc.element('extraAliquotPlate').getElementsByClassName('selectable');
		for (var wellNo = 0; wellNo < wells.length; wellNo++)
		{
			Events.addEventHandler(wells[wellNo], 'click', rnaqc.wellOnClick);
			Events.addEventHandler(wells[wellNo], 'change', rnaqc.extraVolumeOnChange);
		}
		
	}

	rnaqc.wellOnClick = function(event)
	{
		var wellId = Data.int(event.currentTarget, 'well-id');
		var well = rnaqc.getBioWellById(wellId);
		
		var url = 'rnaqc_extra_aliquot.jsp?ID='+App.getSessionId();
		url += '&well='+wellId;
		url += '&rnaqc='+encodeURIComponent(well.bioMaterial.name);
		Dialogs.openPopup(url, 'EditExtraAliquot', 450, 300);	
	}
	
	rnaqc.extraVolumeOnChange = function(event)
	{
		var wellId = Data.int(event.currentTarget, 'well-id');
		var well = rnaqc.getBioWellById(wellId);
		
		Doc.addOrRemoveClass(event.currentTarget, 'has-extra', well.extraVolume);
		event.currentTarget.innerHTML = well.extraVolume+THIN_SPACE+'µl';
		event.currentTarget.title = Strings.encodeTags(well.bioMaterial.name + '\n'+well.comment);
	}
	
	rnaqc.getBioWellById = function(wellId)
	{
		if (!bioplate) return null;
		for (var wellNo = 0; wellNo < bioplate.bioWells.length; wellNo++)
		{
			var well = bioplate.bioWells[wellNo];
			if (well.id == wellId) return well;
		}
		return null;
	}
	
	rnaqc.bioplateOnClick = function()
	{
		var plateId = Data.int('bioplate.name', 'item-id');
		if (!plateId) return;
		Items.viewItem('BIOPLATE', plateId);
	}
	
	rnaqc.validateStep1 = function(event)
	{
		var valid = true;
		
		valid &= Wizard.isValid('csv');
		valid &= Wizard.isValid('qcRunDate');

		if (!valid) event.preventDefault();
	}

	rnaqc.extraAliquotOnClick = function()
	{
		Doc.showHide('extraAliquotPlateSection', Doc.element('extraAliquot.Yes').checked);
	}
	
	rnaqc.buildExtraAliquotPlate = function(plate)
	{
		var html = '';
		html += '<table class="plate">';
		html += '<tr class="header"><th></th>';
		for (var colNo = 0; colNo < plate.geometry.columns; colNo++)
		{
			html += '<th>'+(colNo+1)+'</th>';
		}
		html += '</tr>';
		var wells = plate.bioWells;
		var wellNo = 0;
		for (var rowNo = 0; rowNo < plate.geometry.rows; rowNo++)
		{
			html += '<tr class="row-'+rowNo+'">';
			html += '<th>'+Reggie.wellToAlpha(rowNo)+'</th>';
			for (var colNo = 0; colNo < plate.geometry.columns; colNo++)
			{
				var well = wells[wellNo];
				well.extraVolume = 0;
				well.comment = '';

				if (well.bioMaterial)
				{
					html += '<td id="well-'+well.id+'"';
					html += ' class="well selectable col-'+colNo+'"';
					html += ' title="'+Strings.encodeTags(well.bioMaterial.name)+'" data-well-id="'+well.id+'"';
					html += '>0'+THIN_SPACE+'µl</td>';
				}
				else
				{
					html += '<td class="well empty col-'+colNo+'"></td>';
				}
				wellNo++;
			}
			html += '</tr>';
		}
		return html;
	}
	
	rnaqc.submit = function()
	{
		Doc.hide('wizard-status');
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.csv = frm.csv.value;
		submitInfo.gxdStd = frm.gxdstd.value;
		submitInfo.gxdHi = frm.gxdhi.value;
		submitInfo.pdf = frm.pdf.value;
		submitInfo.QCRunDate = frm.qcRunDate.value;
		submitInfo.QCOperator = frm.qcOperator.value;
		submitInfo.comments = frm.comments.value;
		var extra = [];
		submitInfo.extraAliquots = extra;
		
		if (Doc.element('extraAliquot.Yes').checked)
		{
			for (var wellNo = 0; wellNo < bioplate.bioWells.length; wellNo++)
			{
				var well = bioplate.bioWells[wellNo];
				if (well.extraVolume > 0)
				{
					var tmp = {};
					tmp.id = well.bioMaterial.id;
					tmp.volume = well.extraVolume;
					tmp.comment = well.comment;
					extra[extra.length] = tmp;
				}
			}
		}
		
		var url = '../RnaQc.servlet?ID='+App.getSessionId();
		url += '&cmd=ImportCaliperResults';
			
		Wizard.showLoadingAnimation('Importing RQS values...');
		Wizard.asyncJsonRequest(url, rnaqc.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	rnaqc.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	return rnaqc;
}();

Doc.onLoad(RnaQcImport.initPage);

