var ImportPlates = function()
{
	var importer = {};
	var debug = 0;
	
	var prefixIsValid = false;
	
	var importData;
	
	// Page initialization
	importer.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', importer.validateStep1);

		Events.addEventHandler('prefix', 'change', importer.prefixOnChange);
		Events.addEventHandler('prefix', 'keypress', Wizard.focusOnEnter);
		Wizard.initFileSelectionField('importfile');
		Events.addEventHandler('importfile', 'change', importer.importFileOnChange);
		Events.addEventHandler('sheet', 'change', importer.worksheetOnChange);
		
		Events.addEventHandler('plateNames', 'change', importer.displayAliquotsOnPlate);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', importer.submit);

		Plate.init(9, 9, null, WellPainter);
		Plate.paint(Plate.getWells());
		window.addEventListener('beforeunload', importer.confirmLeaveWizard);
//		Wizard.keepSessionAlive();
		
		Doc.show('step-1');
		Doc.show('gocancel');
		Doc.show('goregister');
		Events.sendChangeEvent('prefix');
	}
	
	/**
		Ask for confirmation to abort if RNA has been placed on the plate.
	*/
	importer.confirmLeaveWizard = function(event)
	{
		if (Plate.getUsed().length > 0)
		{
			event.preventDefault();
		}
	}

	importer.prefixOnChange = function()
	{
		var frm = document.forms['reggie'];
		Wizard.setInputStatus('prefix');
		
		var prefix = frm.prefix.value.toUpperCase().replace(/_/g, '');
		prefixIsValid = false;
		if (prefix.length < 2)
		{
			Wizard.setInputStatus('prefix', 'invalid', 'The prefix should have at least 2 characters.');
			frm.prefix.focus();
			return;
		}
		
		frm.prefix.value = prefix+'_';
		Wizard.setInputStatus('prefix', 'valid');
		prefixIsValid = true;
		
		importer.prevalidateImportFile();
	}
	
	importer.importFileOnChange = function()
	{
		Doc.hide('worksheetSection');
		Doc.hide('worksheetSection.help');
		importer.prevalidateImportFile();
	}
	
	importer.worksheetOnChange = function()
	{
		importer.prevalidateImportFile();
	}
	
	importer.validateStep1 = function(event)
	{
		if (!prefixIsValid || !Wizard.isValid('importfile'))
		{
			event.preventDefault();
		}
	}

	importer.prevalidateImportFile = function()
	{
		Doc.hide('platePreview');
		Wizard.hideWizardStatus();
		Wizard.hideGoNextConfirmation();
		
		var frm = document.forms['reggie'];
		var path = frm.importfile.value;
		if (!prefixIsValid || path == '') return;
		
		Wizard.setInputStatus('importfile');
		Data.set('importfile', 'valid', 0);

		var url = '../Import.servlet?ID='+App.getSessionId();
		url += '&cmd=PreValidateRNAImportFile';
		url += '&path='+encodeURIComponent(path);
		url += '&prefix='+encodeURIComponent(frm.prefix.value);
		url += '&sheet='+encodeURIComponent(frm.sheet.value);
		
		Wizard.showLoadingAnimation('Validating ' + Strings.encodeTags(path));
		Wizard.asyncJsonRequest(url, importer.onImportFileValidated, 'POST');
	}

	importer.onImportFileValidated = function(response)
	{
		var frm = document.forms['reggie'];

		Wizard.setInputStatus('importfile');
		Wizard.showFinalMessage(response.messages);
		
		importData = response.importFile;

		var sheetNames = importData.sheetNames;
		if (sheetNames && sheetNames.length)
		{
			var currentSheet = frm.sheet.value;
			frm.sheet.length = 0;
			for (var sNo = 0; sNo < sheetNames.length; sNo++)
			{
				var sheet = sheetNames[sNo];
				frm.sheet[frm.sheet.length] = new Option(sheet, sheet, false, currentSheet == sheet);
			}
			if (frm.sheet.length > 1) 
			{
				Doc.show('worksheetSection');
				Doc.show('worksheetSection.help');
			}
		}
		
		var plates = importData.plates;
		frm.plateNames.length = 0;
		
		if (plates && plates.length)
		{
			for (var pNo = 0; pNo < plates.length; pNo++)
			{
				var plate = plates[pNo];
				var text = plate.name;
				if (plate.error) 
				{
					text += ' (' + plate.error + ')';
				}
				else
				{
					text += ' (';
					text += plate.id ? 'existing box; ' : 'new box; ';
					text += plate.aliquots.length + ' aliquots)';
				}
				frm.plateNames[frm.plateNames.length] = new Option(text);
			}
		}
		
		if (importData.valid)
		{
			Wizard.setInputStatus('importfile', importData.warnings > 0 ? 'warning' : 'valid');
			Data.set('importfile', 'valid', 1);
			if (importData.warnings > 0)
			{
				Wizard.showGoNextConfirmation(true, 'Check to verify registration of plate with warnings.');
			}
		}
		else
		{
			Wizard.setInputStatus('importfile', 'invalid');
		}
		
		importer.displayAliquotsOnPlate();
	}

	importer.displayAliquotsOnPlate = function()
	{
		Doc.element('filename').innerHTML = Strings.encodeTags(importData.filename);
		Doc.element('numPlates').innerHTML = importData.numPlates;
		Doc.element('numAliquots').innerHTML = importData.numAliquots;
		Doc.element('numErrors').innerHTML = importData.errors;
		Doc.element('numWarnings').innerHTML = importData.warnings;
		
		Doc.element('importPossible').innerHTML = importData.valid ? '<img src="../images/ok.png">Import is possible!' : '<img src="../images/error.png">Import is not possible!';
		
		Doc.show('platePreview');
		var frm = document.forms['reggie'];
		var plateIndex = frm.plateNames.selectedIndex;
		
		var wells = Plate.getWells();
		Plate.clear(wells);
		
		if (plateIndex >= 0 && importData.plates.length > plateIndex)
		{
			var plate = importData.plates[plateIndex];
			var aliquots = plate.aliquots;
			for (var aNo = 0; aNo < aliquots.length; aNo++)
			{
				var aliquot = aliquots[aNo];
				var well = Plate.getWellByLocation(aliquot.location);
				if (well && !well.extract) 
				{
					if (plate.error && !aliquot.error) aliquot.error = plate.error;
					well.setExtract(aliquot);
				}
			}
			var existingAliquots = plate.existingAliquots;
			for (var eNo = 0; eNo < existingAliquots.length; eNo++)
			{
				var aliquot = existingAliquots[eNo];
				var well = Plate.getWellByLocation(aliquot.location);
				if (well && !well.extract) 
				{
					//if (plate.error && !aliquot.error) aliquot.error = plate.error;
					well.setExtract(aliquot);
				}
			}
		}
		Plate.paint(wells);
	}
	
	importer.submit = function()
	{
		var frm = document.forms['reggie'];
		var path = frm.importfile.value;

		var url = '../Import.servlet?ID='+App.getSessionId();
		url += '&cmd=ImportRNAFromFile';
		url += '&path='+encodeURIComponent(path);
		url += '&prefix='+encodeURIComponent(frm.prefix.value);
		url += '&sheet='+encodeURIComponent(frm.sheet.value);
		
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, importer.submissionResults, 'POST');
	}
	
	importer.submissionResults = function(response)
	{
		window.removeEventListener('beforeunload', importer.confirmLeaveWizard);
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	importer.initElements = function(element, autoInit)
	{
		if (autoInit == 'plate-col')
		{
			Events.addEventHandler(element, 'click', importer.toggleColumn);
			Events.addEventHandler(element, 'mouseover', importer.highlightColumn);
			Events.addEventHandler(element, 'mouseout', importer.highlightColumn);
		}
		else if (autoInit == 'plate-row')
		{
			Events.addEventHandler(element, 'click', importer.toggleRow);
			Events.addEventHandler(element, 'mouseover', importer.highlightRow);
			Events.addEventHandler(element, 'mouseout', importer.highlightRow);
		}
		else if (autoInit == 'plate-well')
		{
			Events.addEventHandler(element, 'click', importer.toggleWell);
		}
	}

	// Toggle the selected status of a single well
	importer.toggleWell = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'col');
		var well = Plate.getWell(row, column);
		Plate.toggleSelected([well]);
	}

	// Toggle the selected status of a complete row
	importer.toggleRow = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		Plate.toggleSelected(Plate.getRow(row));
	}

	// Toggle the selected status of a complete column
	importer.toggleColumn = function(event)
	{
		var column = Data.int(event.currentTarget, 'col');
		Plate.toggleSelected(Plate.getColumn(column));
	}
	
	// Highlight enable/disable all wells in a column
	importer.highlightColumn = function(event)
	{
		var column = Data.int(event.currentTarget, 'col');
		var on = event.type == 'mouseover';
		
		Doc.addOrRemoveClass(event.currentTarget, 'highlight-column', on);
		var wells = Plate.getColumn(column);
		Plate.setHighlight(wells, 'highlight-column', on);
	}

	// Highlight enable/disable all wells in a row
	importer.highlightRow = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var on = event.type == 'mouseover';
		
		Doc.addOrRemoveClass(event.currentTarget, 'highlight-row', on);
		var wells = Plate.getRow(row);
		Plate.setHighlight(wells, 'highlight-row', on);
	}
	
	return importer;
}();

Doc.onLoad(ImportPlates.initPage);
Doc.addElementInitializer(ImportPlates.initElements);

var WellPainter = function()
{
	var painter = {};
	
	// Add class indicators for replicates and QC assigned wells
	painter.getClassNameForWell = function(well)
	{
		var cls = '';
		if (well.extract)
		{
			cls += ' primary';
			cls += well.extract.id ? ' existing': ' bg-green';
		}
		if (well.column == Plate.columns-1) cls += ' last-child';
		return cls;
	}
	
	painter.getWellText = function(well)
	{
		var text = '';
		if (well.extract)
		{
			// The well contains RNA
			var aliquot = well.extract;
			text += '<div class="name">'+Reggie.softLineBreaks(Strings.encodeTags(aliquot.name || aliquot.prefix))+'</div>';
			
			if (aliquot.id)
			{
				// Nothing
			}
			else
			{
				text += '<div class="volumes"' + (aliquot.NDConcF ? ' title="NDConc: ' + Reggie.formatNumber(aliquot.NDConcF, 'ng/µl', 1, 1) + '"' : '') + '>';
				text += '<span class="quantity">'+(aliquot.quantityF ? Reggie.formatNumber(aliquot.quantityF*1000, 'ng', 1, 1) : (aliquot.NDConcF ? Reggie.formatNumber(aliquot.NDConcF, 'ng/µl', 1, 1) : '-')) + '</span>';
				text += '<span class="volume">'+(aliquot.volume ? Reggie.formatNumber(aliquot.volume, 'µl', 1, 1) : '-') + '</span>';
				text += '</div>';
				
				text += '<div class="rnaqc">';
				if (aliquot.CA_RQS) 
				{
					text += 'RQS: ' + Strings.encodeTags(aliquot.CA_RQS);
				}
				else if (aliquot.BA_RIN)
				{
					text += 'RIN: ' + Strings.encodeTags(aliquot.BA_RIN);
				}
				text += '</div>';
			}
			if (aliquot.comment)
			{
				text += '<div class="comment">'+Strings.encodeTags(aliquot.comment)+'</div>';
			}
		}
		else
		{
			text = 'empty';
		}
		return text;
	}
	
	return painter;
}();
